;+ 
; NAME:
; mike_fit2darc   
;     Version 1.1
;
; PURPOSE:
;  To fit the arc lines identified in mike_fitarc as a fucntion of
;  their y-centroid and order number.  The main routine is in
;  mike_fit2darc_work.  The fit is a simple least-squares with one
;  round of rejection.
;
; CALLING SEQUENCE:
;   
;  mike_fit2darc, mike
;
; INPUTS:
;   mike     -  MIKE structure
;   setup    -  Integer defining setup
;   obj_id   -  Object identifier
;   [side]   -  Blue (1), Red (2), or both [1,2L]    (Default: [1,2L])
;
; RETURNS:
;
; OUTPUTS:
; A fits file containing the 2D solution.  Named something like
; 'Arcs/Fits/Arc_mb0539_fit2D.fits' 
;
; OPTIONAL KEYWORDS:
;   NOCOEFF   - Number of coefficients to use in the x-direction
;               (default: 6 for blue, 7 for red)
;   NYCOEFF   - Number of coefficients to use in the y-direction
;               (default: 4)
;   /CLOBBER  - Overwrite any previous solution
;   /DEBUG    - debug
;   /CHKRES   - Plot the residuals
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   mike_fit2darc, mike, 1.0
;
; PROCEDURES/FUNCTIONS CALLED:
;  mike_fit2darc_work
;
; REVISION HISTORY:
;   28-Feb-2003 Written by SB
;   18-Apr-2003 Revised by JXP
;-
;------------------------------------------------------------------------------


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function mike_fit2darc_work, arcfil, setup, side, nycoeff=nycoeff, $
                             nocoeff=nocoeff, CLOBBER=clobber, $
                             DEBUG=debug, CHKRES=chkres, out_str=out_str
                   
;
  if  N_params() LT 3  then begin 
      print,'Syntax - ' + $
        'rslt = mike_fit2darc_work( arcfil, setup, side, nycoeff= ) [v1.1]'
      return, -1
  endif

   if NOT keyword_set(nycoeff) then nycoeff = 4
   if keyword_set(nocoeff) then flg_ocoeff = 1L else flg_ocoeff = 0L

   ;; Nocoeff
   if flg_ocoeff NE 1 then begin
       case side of 
           1: nocoeff=6         ; Blue
           2: nocoeff=7         ; Red
           else: return, -1
       endcase
   endif

;  Check for outfil
   outfil = mike_getfil('arc_2Dfit', SUBFIL=arcfil, /name, CHKFIL=chkf)
   if chkf NE 0 and not keyword_set( CLOBBER ) then begin
       print, 'mike_fit2darc: Arc fit file exists. ' + $
         'Continuing..'
       return, outfil
   endif

; Order structure
   ordr_str = mike_getfil('ordr_str', setup, SIDE=side)

; Open arc 1D fit
   arc_info = mike_getfil('arc_fit', subfil=arcfil, /name)
   restore, arc_info

   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 
; SETUP THE DATA

   gd_ordr = where(sv_lines.nlin NE 0, ngd_ordr)
   print, 'mike_fit2darc: Setting up the values (normalizing)'
   npix = round(total(sv_lines.nlin))
   omsk = lonarr(npix)
   t = dblarr(npix)
   ;; PIX AND WV
   cnt = 0L
   for j=0L,ngd_ordr-1 do begin
       ;; Dummy indx
       i = gd_ordr[j]
       ;; PIX
       if j EQ 0 then all_pix = [sv_lines[i].pix[0:sv_lines[i].nlin-1]] $
       else all_pix = [all_pix,sv_lines[i].pix[0:sv_lines[i].nlin-1]]
       ;; WV
       if j EQ 0 then all_wv = [sv_lines[i].wv[0:sv_lines[i].nlin-1]] $
       else all_wv = [all_wv,sv_lines[i].wv[0:sv_lines[i].nlin-1]]
       ;; Order #
       t[cnt:cnt+sv_lines[i].nlin-1] = ordr_str[i].order
       cnt = cnt + sv_lines[i].nlin
   endfor

   nrm = dblarr(2)
   ;; NORMALIZE PIX
   mnx = min(all_pix, MAX=mxx)
   nrm[0] = 0.5 * (mnx + mxx)
   nrm[1] = mxx - mnx
   pix_nrm = 2. * (all_pix - nrm[0])/nrm[1]

   ;; NORMALIZE ORDER
   nrmt = dblarr(2)
   mnx = min(t, MAX=mxx)
   nrmt[0] = 0.5 * (mnx + mxx)
   nrmt[1] = mxx - mnx
   t_nrm = 2. * (t - nrmt[0])/nrmt[1]
   
   invvar = replicate(1., npix)

;  Setup the Functions
   work2d = dblarr(npix,nycoeff*nocoeff)
   worky = flegendre(pix_nrm[*], nycoeff)
   workt = flegendre(t_nrm[*], nocoeff)
   
   for i=0,nocoeff-1 do begin
       for j=0,nycoeff-1 do begin
           work2d[*,j*nocoeff+i] = worky[*, j] * workt[*,i]
       endfor
   endfor

   ;; Do the matrix algebra
   work2di = transpose(work2d * (invvar[*] # replicate(1,nocoeff*nycoeff)))
   alpha = work2di # work2d
   beta = work2di # (alog10(all_wv[*]))
   choldc, alpha, p
   res = cholsol(alpha,p,beta, /double)
   wv_mod = dblarr(npix)
   wv_mod[*] = work2d # res

   ;; Get RMS
   gd_wv = where(invvar GT 0.0, ngd)
   msk = bytarr(npix)
   msk[gd_wv] = 1B
   ;; REJECT
   djs_iterstat, (wv_mod-alog10(all_wv)), sigrej=2.5, mask=msk
   gd = where(msk EQ 1B, complement=bad, ncomplement=nbad)

   ;; RESET invvar
   if nbad NE 0 then begin
       print, 'mike_fit2darc_work: Rejecting ', $
         nbad, ' of ', n_elements(all_wv), ' lines'
       invvar[bad] = 0.
   endif

   ;; Do the matrix algebra
   work2di = transpose(work2d * (invvar[*] # replicate(1,nocoeff*nycoeff)))
   alpha = work2di # work2d
   beta = work2di # (alog10(all_wv[*]))
   choldc, alpha, p
   res = cholsol(alpha,p,beta, /double)
   wv_mod = all_wv * 0.0
   wv_mod[*] = work2d # res


   ;; Finish
   gd_wv = where(invvar GT 0.0, ngd)
   resid = 10^wv_mod[gd_wv] - all_wv[gd_wv]
   fin_rms = sqrt( total( resid^2 ) / float(ngd))
   print, 'mike_fit2darc: RMS = ', fin_rms, ' Ang'
;   if keyword_set(CHKRES) then x_splot, all_wv[gd_wv], resid, psym1=1, /block
   if keyword_set(CHKRES) then x_splot, resid, /block


;  OUTPUT
   out_str = { $
               nrm: nrm, $
               nrmt: nrmt, $
               ny: nycoeff, $
               no: nocoeff, $
               res: res }
   
   ;;  OUTPUT
   print, 'mike_fit2darc: Writing '+outfil
   mwrfits, out_str, outfil, /create

   head = xheadfits(arcfil, /silent)
   sz = lonarr(2)
   sz[0] = sxpar(head,'NAXIS1')
   sz[1] = sxpar(head,'NAXIS2')

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;; QA
   qafil = mike_getfil('qa_arc2dfit', setup, SUBFIL=arcfil) 
   x_psopen, qafil, /maxs
   clr = getcolor(/load)

   ;; NORMALIZE PIX
   all_pix = dindgen(round(sz[1]/8.))*8.
   npix = n_elements(all_pix)
   pix_nrm = 2. * (all_pix - nrm[0])/nrm[1]
   worky = flegendre(pix_nrm[*], nycoeff)
   
   ;; Main plot
   !p.multi=[0,1,1] 
   mn = min(10^wv_mod, max=mx)
   plot, [0.], [0.], color=clr.black, $
     background=clr.white, charsize=1.5, xrange=[mn-30., mx+30.], $
     yrange=[0.,sz[1]], xstyle=1, ystyle=1, ytitle='Row', $
     xtitle='Wavelength', xmargin=[11,2], ymargin=[5,2], /nodata

   nordr = n_elements(ordr_str)
   for jj=0L,nordr-1 do begin
       ;; NORMALIZE ORDER
       ii = ordr_str[jj].order
       tsub = replicate(float(ii), npix)
       t_nrm = 2. * (tsub - nrmt[0])/nrmt[1]
       
       ;; work2d and wv
       work2d = dblarr(npix,nycoeff*nocoeff)
       workt = flegendre(t_nrm[*], nocoeff)
       
       for i=0,nocoeff-1 do begin
           for j=0,nycoeff-1 do begin
               work2d[*,j*nocoeff+i] = worky[*, j] * workt[*,i]
           endfor
       endfor
       
       wv = dblarr(npix)
       wv[*] = work2d # out_str.res
       
       mn = min(10^wv, max=mx)
;       all_x = (ordr_str[jj].lhedg + $
;         ordr_str[jj].rhedg) / 2.

       oplot, 10^wv, all_pix, color=clr.blue
       
       ;; Resid
       pts = where(t EQ ii, npts)
       if npts NE 0 then begin
           nlin = sv_lines[jj].nlin
           sres = 10^wv_mod[pts] - all_wv[pts] 
           oplot, 10^wv_mod[pts] + sres*500., $
             sv_lines[jj].pix[0:nlin-1],  $
             psym=1, color=clr.black
           ;; RMS
;           rms = sqrt( total( sres^2 ) / float(nlin))
       endif
           
       ;; Label
      xyouts, 0.5, 0.96, 'Arc 2D FIT (Res x500) nx='+strtrim(nocoeff,2)+' ny='+$
        strtrim(nycoeff,2)+' RMS='+string(fin_rms,format='(f6.4)')+'Ang', $
        color=clr.black, charsize=1.5, /normal, alignment=0.5
   endfor
           

   ;;;;;;;;;;
   ;; Individual plots
   nordr = n_elements(ordr_str)
   !p.multi=[0,3,2]
   for jj=0L,nordr-1 do begin
       ;; NORMALIZE ORDER
       ii = ordr_str[jj].order
       tsub = replicate(float(ii), npix)
       t_nrm = 2. * (tsub - nrmt[0])/nrmt[1]
       
       ;; work2d and wv
       work2d = dblarr(npix,nycoeff*nocoeff)
       workt = flegendre(t_nrm[*], nocoeff)
       
       for i=0,nocoeff-1 do begin
           for j=0,nycoeff-1 do begin
               work2d[*,j*nocoeff+i] = worky[*, j] * workt[*,i]
           endfor
       endfor
       
       wv = dblarr(npix)
       wv[*] = work2d # out_str.res
       
       mn = min(10^wv, max=mx)
       plot, all_pix, 10^wv, color=clr.black, $
         background=clr.white, charsize=1.5, yrange=[mn, mx], $
         xrange=[0.,sz[1]], xstyle=1, ystyle=1, xtitle='Row', $
         ytitle='Wavelength', xmargin=[11,2], ymargin=[5,1], /nodata
           
       ;; Fit
       oplot, all_pix, 10^wv, color=clr.blue
       
       ;; Resid
       pts = where(t EQ ii, npts)
       rms = 9.99
       if npts NE 0 then begin
           nlin = sv_lines[jj].nlin
           sres = 10^wv_mod[pts] - all_wv[pts] 
           oplot, sv_lines[jj].pix[0:nlin-1],  10^wv_mod[pts] + sres*100., $
             psym=1, color=clr.black
           ;; Rej
           rej = where(invvar[pts] LE 0., nrej, ncomplement=nnorej, $
                       complement=norej)
           if nrej NE 0 then $
             oplot, [(sv_lines[jj].pix[0:nlin-1])[rej]],  $
             [(10^wv_mod[pts] + sres*100.)[rej]], $
             psym=2, color=clr.red
           ;; RMS
           if nnorej NE 0 then $
             rms = sqrt( total( sres[norej]^2 ) / float(nnorej-1)) $
             else rms = 9.99
       endif
           
       ;; Label
       if side EQ 2 then ylbl = mn + (mx-mn)*0.08*(findgen(5)+1) $
       else ylbl = mx - (mx-mn)*0.08*(findgen(5)+1) 
       
       xyouts, sz[1]*0.05, ylbl[0], 'Order = '+strtrim(ii,2), $
         color=clr.black, charsize=1.5
       dwv = abs(10^wv[0]-10^wv[npix-1])/float(all_pix[npix-1])
       xyouts, sz[1]*0.05, ylbl[2], '!9Dl!X = '+string(dwv,format='(f6.4)'), $
         color=clr.black, charsize=1.5
       xyouts, sz[1]*0.05, ylbl[1], 'RMS(pix) = '+$
         string(rms/dwv,format='(f4.2)'), color=clr.black, charsize=1.5
   endfor
           
   x_psclose
   !p.multi=[0,1,1]
   spawn, 'gzip -f '+qafil

   return, outfil

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro mike_fit2darc, mike, setup, obj_id, side, CLOBBER=clobber, $
                   nycoeff=nycoeff, nocoeff=nocoeff, DEBUG=debug, CHKRES=chkres
;
  if  N_params() LT 3  then begin 
      print,'Syntax - ' + $
        'mike_fit2darc, mike, setup, obj_id, [side], nycoeff=, nocoeff=' + $
        '/CLOBBER, /CHKRES, /DEBUG [v1.1]'
      return
  endif

   if NOT keyword_set(nycoeff) then nycoeff = 4
   if not keyword_set( SIDE ) then side = [1L,2L]
   if keyword_set(nocoeff) then flg_ocoeff = 1L else flg_ocoeff = 0L

; Label
  if setup GE 10 then c_s = strtrim(setup,2) else c_s = '0'+strtrim(setup,2) 

; Loop on side
  for kk=0L,n_elements(side)-1 do begin
      qq = side[kk]
      ;; SIDE
      if qq EQ 1 then print, 'mike_fitarc: Fitting BLUE arc' $
      else print, 'mike_fitarc: Fitting RED arc'

      ;; Nocoeff
      if flg_ocoeff NE 1 then begin
          case qq of 
              1: nocoeff=6 ; Blue
              2: nocoeff=7 ; Red
              else: stop
          endcase
      endif

      ;; Grab all obj indices
      indx = where(mike.flg_anly NE 0 AND mike.side EQ qq AND $
                   mike.obj_id EQ obj_id AND mike.setup EQ setup AND $
                   strtrim(mike.type,2) EQ 'OBJ', nindx)
      if nindx EQ 0 then begin
          print, 'mike_fitarc: No Obj found!  Returning' 
          continue
      endif


      arcfil_all= mike[indx].arc_fil
      asort = sort(arcfil_all)
      auniq = uniq(arcfil_all[asort])

  
      ;; LOOP
      for mm=0L,n_elements(auniq)-1 do begin
          idx = indx[asort[auniq[mm]]]

          rslt = mike_fit2darc_work( mike[idx].arc_fil, $
                                   setup, qq, nycoeff=nycoeff, CHKRES=chkres, $
                                   nocoeff=nocoeff, OUT_STR=out_str, $
                                   CLOBBER=clobber, DEBUG=debug)
          ;; Trouble?
          if size(rslt,/tname) NE 'STRING' then stop
      endfor
      
  endfor
;

   print, 'mike_fit2darc: All done'
   return

end

