;+ 
; NAME:
; mike_fitarc   
;     Version 1.1
;
; PURPOSE:
;  To identify and centroid arc lines in each order.  There is
;  actually no fitting done at this stage other than to reject bad
;  lines.   The main program calls mike_fitarc_work as its driver.
;  The algorithm mike_fitarc_work does the following:
;
;    1) Input the arc image from mike_mkarc 
;    2) Input an archived arc solution appropriate to the side
;    3) Extract 1D (boxcar) spectra down each order :: extract_arc
;    4) Cross correlate (FFT) against the archived 1D arc spectrum,
;    this gives the order number and the pixel offset
;    5) Automatically identify a set of lines (x_templarc)
;    6) Perform a low order fit to these lines
;    7) Reidentify all lines with this fit and refit 
;    8) Write arc solutions (one per order) to a fits file 
;    9) If the orders extend beyond the archived solution, attempt to
;    extrapolate to the remaining orders.  The idea is to use the
;    known wavelengths from the good orders to extrapolate a guess at
;    the solution.  Then use this guess to peak up on the arc lines in
;    these additional orders.
;
; CALLING SEQUENCE:
;   
;  mike_fitarc, mike, setup, obj_id, [side], /INTER, LINLIST=, /CHK, /CLOBBER,
;  SIGREJ=, /DEBUG, IORDR=, /PINTER 
;
; INPUTS:
;   mike     -  MIKE structure
;   setup    -  Integer defining setup
;   obj_id   -  Object identifier
;   [side]   -  Blue (1), Red (2), or both [1,2L]    (Default: [1,2L])
;
; RETURNS:
;
; OUTPUTS:
;  IDL fit file (one per order)  (e.g. Arcs/ArcECH_##fit.idl)
;
; OPTIONAL KEYWORDS:
;   /PINTER   - Perform fit for pre-identified lines
;   /INTER    - Identify lines interactively and then fit
;   LINLIST   -  Arc line list (default: $XIDL_DIR/Spec/Arcs/Lists/mike_thar.lst
;   /CHK      - Manually check steps along the way
;   /DEBUG    - Debugging
;   SIGREJ=   - Rejection sigma for outliers in arc line fitting
;              (default: 2.)
;   IORDR     - Initial order for analysis
;   /CLOBBER  - Overwrite previous fits
;   SHFTPRM=  - Fit structure for shifting the orders of the arc
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   mike_fitarc, mike, 1, 1
;
;
; PROCEDURES/FUNCTIONS CALLED:
;   mike_crossarc
;   mike_fitarc_ps
;   mike_fitarc_work
;   extract_arc
;   x_templarc
;
; REVISION HISTORY:
;   12-Aug-2002 Written by JXP
;   21-Aug-2002 Streamlined + Added ps output
;   01-Feb-2003 Polished (JXP)
;   01-Jan-2004 Added a guess at unmatched orders (SB)
;-
;------------------------------------------------------------------------------

pro mike_fitarc_ps, svdecomp, filnm, WV=wv, FIT=fit, REJPT=rejpt,$
                      ORDR=ordr, RMS=rms, FORDR=fordr, DWV=dwv

  clr = getcolor(/load)
  mn = min(wv, max=mx)
  xlbl = mn + (mx-mn)*0.05

  ;; All points
  plot, wv, (fit-wv)/dwv, psym=1, $
    charsize=1.8, $
    background=clr.white, color=clr.black, $
    xtitle='Wavelength (Ang)', ytitle='Residual (Pix)', $
    xmargin=[11,1], ymargin=[5,1], yrange=[-0.7, 0.7], xrange=[mn-1,mx+1.], $
    xstyle=1, ystyle=1, xtickinterval=(fix(wv[0]/1000.)-1)*10
  ;; MASK 
  if rejpt[0] NE -1 then $
    oplot, [wv[rejpt]], [(fit[rejpt]-wv[rejpt])/dwv], psym=2, color=clr.red
  ;; Order

  ;; 
  oplot, [-9e9, 9e9], [0., 0.], linestyle=1, color=clr.blue

  xyouts, xlbl, 0.6, 'Order = '+string(ordr, FORMAT='(i3)')+$
    '   !9Dl!X = '+string(dwv, FORMAT='(f6.4)'), charsize=1.3
  xyouts, xlbl, 0.48, 'RMS='+string(rms, FORMAT='(f6.4)')+$
    '   Fit Ord='+string(fordr, FORMAT='(i2)'), charsize=1.3
          
  return
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function mike_fitarc_work, arc_fil, setup, side, $
                      INTER=inter, LINLIST=linlist, $
                      CHK=chk, CLOBBER=clobber, SIGREJ=sigrej, DEBUG=debug,$
                      IORDR=iordr, PINTER=pinter, $
                      GUESSARC=guessarc, SHFTPRM=shftprm


;
  if  N_params() LT 3  then begin 
      print,'Syntax - ' + $
        'a = mike_fitarc_work(arc_fil, setup, side, /INTER, IORDR=, /DEBUG, /CHK '
      print, '     SHFTPRM=, /clobber) [v1.1]'
      return, -1
  endif 
  
;  Optional Keywords
  if not keyword_set( SIGREJ ) then sigrej = 2.
  if not keyword_set( TRCNSIG ) then trcnsig = 3.
  if not keyword_set( RADIUS ) then radius = 2.0
  if not keyword_set( LINLIST ) then $
    linlist = getenv('XIDL_DIR')+'/Spec/Arcs/Lists/mike_thar.lst' 
  if not keyword_set( PKWDTH ) then pkwdth = 3L
  if not keyword_set( NOLOG ) then FLOG = 1 else FLOG = 0
;  if not keyword_set( XOFF ) then xoff = 0.

  ;; Read in order structure
  ordr_str = mike_getfil('ordr_str', setup, SIDE=side, fil_nm=ordr_fil)
  nordr = n_elements(ordr_str)
  temp_ordr = ordr_str
  if keyword_set( SHFTPRM ) then $  ; Shift as requesetd
    xoff = mike_shifti(shftprm, OSTR=temp_ordr)

  ;; Set orders
  if side EQ 1 then ostep = 1 else ostep = -1
  if not keyword_set( IORDR ) then begin
      ordrs = [ordr_str[0].order, ordr_str[nordr-1].order]
      flg_ordr = 0
  endif else begin
      ordrs = [iordr, ordr_str[nordr-1].order]
      flg_ordr = 1
  endelse

  ;; OUTFIL
  out_fil = mike_getfil('arc_fit', setup, SIDE=side, $
                        /name, SUBFIL=arc_fil, CHKFIL=chkfil)
  if flg_ordr EQ 0 AND CHKFIL NE 0 AND not keyword_set( CLOBBER ) then begin
      print, 'mike_fitarc: Arc fits file exists!  ' + $
        'Use /CLOBBER to overwrite'
      return, out_fil
  endif 


; Open line list
  x_arclist, linlist, lines
  ;; SORT
  srt = sort(lines.wave)
  lines = lines[srt]

; Grab Arc IMG

  if x_chkfil(arc_fil+'*',/silent) EQ 0 then begin
      print, 'mike_fitarc: Arc ', arc_fil, ' does not exist. ' + $
        'Run mike_procarc!'
      return, -1
  endif
  print, 'mike_fitarc: Reading arc: ', arc_fil
  arc_img = xmrdfits(arc_fil, 0, head, /silent)
  sz_arc = size(arc_img, /dimensions)
  sat_region = 25*sz_arc[1]/2048L



; Open Old file as necessary
  if flg_ordr EQ 1 then restore, out_fil

; Loop on Orders

  ;; MEDWID
  if not keyword_set( MEDWID ) then begin
      if sz_arc[0] EQ 2048L then MEDWID = 3L
      if sz_arc[0] EQ 1024L then MEDWID = 2L
      if sz_arc[0] LT 1024L then MEDWID = 1L
      if not keyword_set( MEDWID ) then stop
  endif
  

  msk = bytarr(sz_arc[1])
  tmp = fltarr(2*MEDWID+1L, sz_arc[1])
  ;; Create output as necessary
  if flg_ordr EQ 0 then begin
      sv_aspec = fltarr(sz_arc[1], 50)
      fittmp = { fitstrct }
      all_arcfit = replicate(fittmp, 50)
      rejtmp = { $
                 ngdf: 0L, $
                 gdfpt: lonarr(100), $
                 gdfpx: dblarr(100), $
                 nrej: 0L, $
                 rejpt: lonarr(100), $
                 rejwv: dblarr(100) $
            }
      rejstr = replicate(rejtmp, 50L)
      lintmp = { $
                 pix: dblarr(90), $
                 wv: dblarr(90), $
                 nlin: 0 $
               }
      sv_lines = replicate(lintmp, 50)
  endif

  ;; Extract the arc (boxcar)
  sv_aspec =   fltarr(sz_arc[1], 50)
  sv_logwv    =   sv_aspec
  sv_aspec[*,0:nordr-1] =  extract_arc(arc_img, temp_ordr)
   
; Open Guess (If not interactive)
  if not keyword_set( INTER ) then begin
      ;; Open archived solutions
      if not keyword_set( GUESSARC ) then begin
        guessarc = mike_getfil('guess_arc', SIDE=side, SZ=sz_arc, /name, $
              chkfil=chkfil)
       if chkfil EQ 0 then $
         guessarc = mike_getfil('guess_arc', SIDE=side, SZ=[1024,2048], /name, $
              chkfil=chkfil)
         if chkfil EQ 0 then begin
           print, "Cannot find arc template: " , guessarc
           return, -1
         endif
      endif
 
         
      mike_crossarc, guessarc, sv_aspec, guess_spec, guess_fit, guess_ordr,$
           ordr_shift, row_shift, chk=chk, sigrej=sigrej


      print, 'Orders offset by ', ostep*ordr_shift, $
            ' And pixel offset by ', row_shift

      ;; Calculate and save the physical order number
      orders_fft =  (lindgen(nordr) + ordr_shift) * ostep + guess_ordr[0] 
      ordr_str.order = orders_fft

      ;; Write new order structure
      mwrfits, ordr_str, ordr_fil, /create, /silent
  endif

      
  svshft = transpose([-99, -999L])
; LOOP on ORDERS to identify and centroid arc lines
  for ii=0, nordr-1 do begin  
      ;; Grab index
      jj = ordr_str[ii].order
      print, 'mike_fitarc: Order ', jj, format='(a, i4, $)'
      igss = where(guess_ordr EQ jj, n_guess)

      ;; Zero out
      lines[*].flg_plt = 0
      
; INTERACTIVE
      if keyword_set(INTER) then begin 
          x_identify, sv_aspec[*,ii], finfit, LINELIST=linlist, MSK=msk, $
            OUTLIN=outlin, XSIZE=xsize, YSIZE=ysize
          all_arcfit[ii] = finfit 
          ngd = n_elements(outlin)
          sv_lines[ii].nlin = ngd
          sv_lines[ii].pix[0:ngd-1] = outlin.pix
          sv_lines[ii].wv[0:ngd-1] = outlin.wave
      endif else begin  
; AUTO or SEMI-AUTO (PINTER)
          ;; Identify guess
          if n_guess NE 1 then begin
            print, 'Not included in guess spectra, skip to next order'
            continue
          endif
          
          ;; Cross-correlate
          step = lindgen(80L) - 40L + row_shift
          corr = c_correlate((sv_aspec[*,ii]<30000.), $
                             (guess_spec[*,igss]<30000.), step, /double)
          mx = max(corr, imx)
          imx = step[imx]
          shft = -imx 
          print, ', Shift', shft, format='(a,i4,$)'

          ;; Check against row_shift ??
          
          ;; Peak sigma
          if side EQ 1 then begin
              ;; There are too few lines in the bluest orderst to use 
              ;;  sigma=50 with some exposure times
              if jj LT 105 then psig = 50. 
              if jj GT 105 AND jj LT 109 then psig = 20.
              if jj GE 109 then psig = 5.
          endif else begin
              ;; There are some issues on the red side too.  These
              ;; will probably be alliviated with a neutral density filter
              if jj GT 40 then psig = 30. $
              else psig = 10.
          endelse

          ;; DEBUG
          if keyword_set( DEBUG ) then stop

          msk = bytarr(sz_arc[1])
          msk[where(sv_aspec[*,ii] GT 0)] = 1B

          ;; SAT
          satpix = (sv_aspec[*,ii] GT 55000)
          sat = where(smooth(1.0*satpix,sat_region) GT 0, nsat) 
          if nsat NE 0 then msk[sat] = 0B

          if ptr_valid(guess_fit[igss].ffit) EQ 0 then begin
            print, 'Template order is not valid' 
            continue
          endif
  
          ;; Auto ID a set of lines using the archived solution and
          ;; the offset
          x_templarc, sv_aspec[*,ii], lines, guess_fit[igss], MSK=msk, $
            SHFT=shft, ALL_PK=all_pk, PKWDTH=pkwdth, FORDR=9, LOG=FLOG, $
            PKSIG=psig
          
        
          ;; Check the number of good lines
          gdfit = where(lines.flg_plt EQ 1, ngd)
          if ngd LE 4 then begin
              if jj LT 103 then begin
                  print, 'mike_fitarc: Insufficient lines for AUTO!!'
                  continue
              endif else begin
                  print, 'mike_fitarc: Only ', ngd, ' lines here in the blue'
                  print, 'mike_fitarc: Fitting with nord = ', ngd-1
                  guess_fit[igss].nord = ngd - 1
                  break
              endelse
          endif
                  
          ;; FIRST (crude) FIT
          tmp_fit = {fitstrct}
          copy_struct, guess_fit[igss], tmp_fit, $
            EXCEPT_TAGS=['FFIT','NRM','RMS']
          tmp_fit.flg_rej = 1 
          tmp_fit.niter = 3 
          tmp_fit.maxrej = 10 
          tmp_fit.minpt = 5
          tmp_fit.hsig = sigrej
          tmp_fit.lsig = sigrej
          tmp_fit.nord = (3 < guess_fit[igss].nord)   ; Set first fit to 3
          fin_fit = tmp_fit
          fin_fit.nord = guess_fit[igss].nord
          
          fit = x_fitrej(lines[gdfit].pix, alog10(lines[gdfit].wave), $
                         FITSTR=tmp_fit, REJPT=rejpt)
          if fit[0] EQ -1 then begin
              print, 'mike_fitarc: AUTO Failed!!'
              stop
          endif
                  
          ;; Grab new lines
          lines.flg_plt = 0
          shft = 0L  ; No shift expected now!
          ;; Final sigma
          if side EQ 1 then fsig = jj LT 103 ? 20. : 5. $
          else fsig = jj GT 45 ? 30. : 10.

          ;; AutoID lines
          x_templarc, sv_aspec[*,ii], lines, tmp_fit, MSK=msk, PKWDTH=pkwdth, $
            FORDR=9, SHFT=shft, /LOG, PKSIG=fsig
          svshft = [svshft, transpose( [jj, round(shft)]) ]

          gdfit = where(lines.flg_plt EQ 1, ngd)
          ;; Adjust order number
          fin_fit.nord = fin_fit.nord < (ngd-1)

          ;; Save gdfit (for QA)
          rejstr[ii].ngdf = ngd
          rejstr[ii].gdfpt[0:ngd-1] = gdfit
          rejstr[ii].gdfpx[0:ngd-1] = lines[gdfit].pix
          
          ;; CHK 
          if keyword_set( CHK ) then begin
              if keyword_set( DEBUG ) then $
                printcol, lines[gdfit].pix, lines[gdfit].wave
              x_prspeaks, sv_aspec[*,ii], lines[gdfit].pix, /block
              if keyword_set( DEBUG ) then stop
          endif
          
          ;; FIT
          if not keyword_set( PINTER ) then begin ;; AUTO FIT
              fit = x_fitrej(lines[gdfit].pix, alog10(lines[gdfit].wave), $
                             FITSTR=fin_fit, REJPT=rejpt, GDPT=gdpt)
              if fit[0] EQ -1 then begin
                  print, 'mike_fitarc: Lowering order by 1'
                  fin_fit.nord = fin_fit.nord - 1
                  fit = x_fitrej(lines[gdfit].pix, alog10(lines[gdfit].wave), $
                                 FITSTR=fin_fit, REJPT=rejpt, GDPT=gdpt)
                  if fit[0] EQ -1 then stop
              endif
              ;; Save lines
              ngd = n_elements(gdpt)
              sv_lines[ii].nlin = ngd
              sv_lines[ii].pix[0:ngd-1] = lines[gdfit[gdpt]].pix
              sv_lines[ii].wv[0:ngd-1] = lines[gdfit[gdpt]].wave
          endif else begin ;; INTER FIT in x_identify
              svlines = lines
              x_identify, sv_aspec[*,ii], id_fit, LINELIST=linlist, $
                INLIN=lines, OUTLIN=outlin, XSIZE=xsize, YSIZE=ysize
              ;;  Parse the rejected lines
              rejpt = where(lines.flg_plt EQ 0 AND svlines.flg_plt EQ 1)
              ;; Save
              ngd = n_elements(outlin)
              sv_lines[ii].nlin = ngd
              sv_lines[ii].pix[0:ngd-1] = outlin.pix
              sv_lines[ii].wv[0:ngd-1] = outlin.wave
              ;; Final auto fitting in log10 space!
              fin_fit.nord = id_fit.nord
              ;; Cant deal with additional rejected lines for now  JXP
              fit = x_fitrej(outlin.pix, alog10(outlin.wave), FITSTR=fin_fit)
          endelse
          
          ;; Rejected points (for QA)
          if rejpt[0] NE -1 then begin
              rejstr[ii].nrej = n_elements(rejpt)
              rejstr[ii].rejpt[0:rejstr[ii].nrej-1] = rejpt
              rejstr[ii].rejwv[0:rejstr[ii].nrej-1] = lines[gdfit[rejpt]].wave
          endif else rejstr[ii].nrej=0L

          ;; Output results of the fit 
          print, ', NLIN=', ngd, format='(a, i4, $)'
          print, ', RMS (Ang,Pix) = ', $
            fin_fit.rms*sv_lines[ii].wv[0]*alog(10.), format='(a,f7.4,$)'
          sv_logwv[*,ii] = x_calcfit(dindgen(sz_arc[1]),fitstr=fin_fit)
          wv = 10^sv_logwv[*,ii]
          if side EQ 1 then dwv = median(wv - shift(wv,1)) else $
              dwv = median(wv - shift(wv,-1))

          print, fin_fit.rms*sv_lines[ii].wv[0]*alog(10.) / dwv,format='(f8.4)'
          all_arcfit[ii] = temporary(fin_fit)
          
      endelse
      ;; OUTPUT
      if keyword_set( DEBUG ) then $
      print, 'mike_fitarc: Writing fit to ', out_fil
      ;; Output to hard drive (IDL file)
      save, sv_aspec, all_arcfit, sv_lines, rejstr, filename=out_fil
  endfor
  
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  ;; Now fill in the rest of the orders where we couldn't find a template
  print, 'Now attempting to extrapolate to unmatched orders'
  print, '   ...wish me luck'

  log_order = alog10(ordr_str.order) # replicate(1,sz_arc[1])
  wave_data  = transpose(sv_logwv[*,0:nordr-1])  
  xy2traceset, log_order, wave_data, tset_wave, invvar=wave_data GT 2.5, $
    ncoeff = 3, yfit=wave_fit, /silent, /double

  wave_fit = transpose(wave_fit)
  
  search_orders = where(wave_data[*,0] LT 2.5, nsearch)
  
  ;;
  ;;  Just copy and paste from above
  ;;  
  for i=0,nsearch -1 do begin
      ii = search_orders[i]
      jj =  ordr_str[ii].order
      print, 'mike_fitarc: Order ', jj, format='(a, i4, $)'
      
      x_wave = dindgen(sz_arc[1])
      tmp_fit.nord = 4
      fin_fit = tmp_fit
      fin_fit.nord = 5
      
      fit = x_fitrej(x_wave, wave_fit[*,ii], FITSTR=tmp_fit, REJPT=rejpt)
      
      ;; Grab new lines   
      lines.flg_plt = 0
      
      ;; Find shift from closest
;      mn = min(abs(jj-svshft[*,0]), imn) 
;      shft = svshft[imn,1]  
;      shft = 0L
      if i NE 0 then shft = svshft

      ;; Final sigma
      if side EQ 1 then fsig = jj LT 103 ? 20. : 5. $
      else fsig = jj GT 45 ? 30. : 10.
      
      ;; Mask
      msk = bytarr(sz_arc[1])
      msk[where(sv_aspec[*,ii] GT 0)] = 1B
      satpix = (sv_aspec[*,ii] GT 55000)
      sat = where(smooth(1.0*satpix,sat_region) GT 0, nsat) 
      if nsat NE 0 then msk[sat] = 0B
      
      anylines = where(lines.wave GT 10^min(fit) AND $
                       lines.wave LT 10^max(fit), nany)
      if nany LT tmp_fit.nord then begin
          print, 'mike_fitarc: Not enough lines to match'
          fin_fit = tmp_fit
          all_arcfit[ii] = temporary(fin_fit)
          continue
      endif  
      
      x_templarc, sv_aspec[*,ii], lines, tmp_fit, MSK=msk, PKWDTH=pkwdth, $
        FORDR=9, SHFT=shft, /LOG, PKSIG=fsig
      svshft = shft

      gdfit = where(lines.flg_plt EQ 1, ngd)
      
      if (ngd LE tmp_fit.nord) then begin
          print, 'mike_fitarc: Not enough good lines.  Punting...' 
          continue
      endif
      
      ;; Save gdfit (for QA)
      rejstr[ii].ngdf = ngd
      rejstr[ii].gdfpt[0:ngd-1] = gdfit
      rejstr[ii].gdfpx[0:ngd-1] = lines[gdfit].pix
          
      ;; Adjust order number
      fin_fit.nord = fin_fit.nord < (ngd-1)
      
      if keyword_set( CHK ) then begin
          if keyword_set( DEBUG ) then $
            printcol, lines[gdfit].pix, lines[gdfit].wave
          x_prspeaks, sv_aspec[*,ii], lines[gdfit].pix, /block
          if keyword_set( DEBUG ) then stop
      endif
      
      ;; FIT
      if not keyword_set( PINTER ) then begin ;; AUTO FIT
          fit = x_fitrej(lines[gdfit].pix, alog10(lines[gdfit].wave), $
                         FITSTR=fin_fit, REJPT=rejpt, GDPT=gdpt)
          if fit[0] EQ -1 then begin
              print, 'mike_fitarc: Lowering order by 1'
              fin_fit.nord = fin_fit.nord - 1
              fit = x_fitrej(lines[gdfit].pix, alog10(lines[gdfit].wave), $
                             FITSTR=fin_fit, REJPT=rejpt, GDPT=gdpt)
              if fit[0] EQ -1 then stop
          endif
          ;; Save lines
          ngd = n_elements(gdpt)
          sv_lines[ii].nlin = ngd
          sv_lines[ii].pix[0:ngd-1] = lines[gdfit[gdpt]].pix
          sv_lines[ii].wv[0:ngd-1] = lines[gdfit[gdpt]].wave
      endif else begin ;; INTER FIT in x_identify
          x_identify, sv_aspec[*,ii], id_fit, LINELIST=linlist, $
            INLIN=lines, OUTLIN=outlin, XSIZE=xsize, YSIZE=ysize
          ngd = n_elements(outlin)
          sv_lines[ii].nlin = ngd
          sv_lines[ii].pix[0:ngd-1] = outlin.pix
          sv_lines[ii].wv[0:ngd-1] = outlin.wave
          ;; Final auto fitting in log10 space!
          fin_fit.nord = id_fit.nord
          fit = x_fitrej(outlin.pix, alog10(outlin.wave), FITSTR=fin_fit)
      endelse
      
      ;; Rejected points (for QA)
      if rejpt[0] NE -1 then begin
          rejstr[ii].nrej = n_elements(rejpt)
          rejstr[ii].rejpt[0:rejstr[ii].nrej-1] = rejpt
      endif else rejstr[ii].nrej=0L

      ;; Output results of the fit 
          print, ', NLIN=', ngd, format='(a, i4, $)'
          print, ', RMS (Ang,Pix) = ', $
            fin_fit.rms*sv_lines[ii].wv[0]*alog(10.), format='(a,f7.4,$)'
      sv_logwv[*,ii] = x_calcfit(dindgen(sz_arc[1]),fitstr=fin_fit)
      wv = 10^sv_logwv[*,ii]
      if side EQ 1 then dwv = median(wv - shift(wv,1)) else $
        dwv = median(wv - shift(wv,-1))

      print, fin_fit.rms*sv_lines[ii].wv[0]*alog(10.) / dwv,format='(f8.4)'
      all_arcfit[ii] = temporary(fin_fit)
      
      ;; OUTPUT
      if keyword_set( DEBUG ) then $
        print, 'mike_fitarc: Writing fit to ', out_fil
      ;; Output
      save, sv_aspec, all_arcfit, sv_lines, rejstr, filename=out_fil
  endfor
  
  ;; QA
;  restore, out_fil
  qafil = mike_getfil('qa_arcfit', setup, SUBFIL=arc_fil) 
  x_psopen, qafil, /maxs
  clr = getcolor(/load)
  !p.multi=[0,3,2]
 
  ;; Plot to ps file  (this does not include edited lines)
  for ii=0L,nordr-1 do begin
      if rejstr[ii].ngdf EQ 0L then continue
      gdfit = rejstr[ii].gdfpt[0:rejstr[ii].ngdf-1]
      ;; Rejpt
      if rejstr[ii].nrej NE 0 then rejpt = rejstr[ii].rejpt[0:rejstr[ii].nrej-1] $
      else rejpt = -1L
      ;; Dwv
      wv = 10^x_calcfit(dindgen(sz_arc[1]),fitstr=all_arcfit[ii])
      if side EQ 1 then dwv = median(wv - shift(wv,1)) else $
        dwv = median(wv - shift(wv,-1))
      ;; Subroutine 
      fit=10^x_calcfit(double(rejstr[ii].gdfpx[0:rejstr[ii].ngdf-1]), $
                         FITSTR=all_arcfit[ii])
      mike_fitarc_ps, WV=lines[gdfit].wave, $
        fit=fit, $
        REJ=rejpt, $
        DWV=dwv, $
        ORDR=ordr_str[ii].order,$
        RMS=all_arcfit[ii].rms*sv_lines[ii].wv[0]*alog(10.) / dwv, $ ; pix
        FORDR=all_arcfit[ii].nord
  endfor

  x_psclose
  !p.multi=[0,1,1]
  spawn, 'gzip -f '+qafil

  ;; Clean up
  if keyword_set(GUESSARC) then delvarx, guessarc

  print, 'mike_fitarc_work: All done!'

  return, out_fil
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro mike_fitarc, mike, setup, obj_id, side, $
                 INTER=inter, LINLIST=linlist, $
                 CHK=chk, CLOBBER=clobber, SIGREJ=sigrej, DEBUG=debug,$
                 IORDR=iordr, PINTER=pinter


;
  if  N_params() LT 3  then begin 
      print,'Syntax - ' + $
        'mike_fitarc, mike, setup, obj_id, [side], /INTER, IORDR=, /DEBUG,'+ $
        '/CHK, SIGREJ=, /CLOBBER, LINLIST= [v1.1]'
      return
  endif 
  
;  Optional Keywords
  if not keyword_set( SIDE ) then side = [1L,2L]

; Loop on side
  for kk=0L,n_elements(side)-1 do begin
      qq = side[kk]
      ;; SIDE
      if qq EQ 1 then print, 'mike_fitarc: Fitting BLUE arc' $
      else print, 'mike_fitarc: Fitting RED arc'

      ;; Grab all obj indices
      indx = where(mike.flg_anly NE 0 AND mike.side EQ qq AND $
                   mike.obj_id EQ obj_id AND mike.setup EQ setup AND $
                   (strtrim(mike.type,2) EQ 'OBJ' OR $
                   strtrim(mike.type,2) EQ 'STD'), nindx)
      if nindx EQ 0 then begin
          print, 'mike_fitarc: No Obj found!  Skipping..' 
          continue
      endif

      arcfil_all= mike[indx].arc_fil
      asort = sort(arcfil_all)
      auniq = uniq(arcfil_all[asort])

      ;; LOOP
      for mm=0L,n_elements(auniq)-1 do begin
 
          arcfil = arcfil_all[asort[auniq[mm]]]
          idx = indx[asort[auniq[mm]]]

          ;; Check for arcfil
          if x_chkfil(arcfil+'*',/silent) EQ 0 then begin
              print, 'mike_fitarc: Use mike_procarc to create arc file first!!'
              continue
          endif

          rslt = mike_fitarc_work( arcfil, setup, qq, INTER=inter, $
                                   LINLIST=linlist,$ 
                                   CHK=chk, CLOBBER=clobber, $
                                   PINTER=pinter, DEBUG=debug, SIGREJ=sigrej, $
                                   IORDR=iordr)
;                                   XOFF=mike[idx].arc_xyoff[0], $ 
          if size(rslt,/tname) NE 'STRING' then stop
      endfor
  endfor

; All done
  print, 'mike_fitarc: All done!'

  return
end
