;+ 
; NAME:
; mike_mkaimg
;     Version 1.1
;
; PURPOSE:
;   Given the 2D solution for the slope of the lines as a function of
;   position this code creates a wavelength image (i.e. assigns a
;   unique wavelength to each pixel in each order).  The xoffset is 
;   input in order to properly determine the edges of each order.  A
;   simple spline interpolation is used to determine the values.
;
;
; CALLING SEQUENCE:
;   
;  mike_mkaimg, mike, setup, [side], /CHK, /CLOBBER
;
; INPUTS:
;   mike     -  MIKE structure
;   setup    -  Integer defining setup
;   obj_id   -  Object identifier
;   [side]   -  Blue (1), Red (2), or both [1,2L]    (Default: [1,2L])
;
; RETURNS:
;
; OUTPUTS:
;  2D wavelength image with name like 'Arcs/Arc_mb0439I.fits'
;
; OPTIONAL KEYWORDS:
;   /CLOBBER  - Overwrite previous image
;   /CHK      - Display the final image
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   mike_mkaimg, mike, setup, obj_id
;
;
; PROCEDURES/FUNCTIONS CALLED:
;  mike_getfil
;  mike_ordermask
;  mike_mkaimg_work
;
; REVISION HISTORY:
;   15-May-2003 Written by SB
;
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function mike_mkaimg_work, arc_fil, setup, side, CHK=chk, CLOBBER=clobber, $
                           BAD_ANLY=bad_anly, SHFTPRM=shftprm
;

  if  N_params() LT 3  then begin 
      print,'Syntax - ' + $
        'rslt = mike_mkaimg_work( arc_fil, setup, side, /CHK, /CLOBBER, SHFTPRM= ) [v1.1]'
      return, -1
  endif 

  ;; Optional keywords
  if not keyword_set( SHFTPRM ) then SHFTPRM = [0., 0.]
  bad_anly = 0
 
  ;; Check for outfil
  out_fil = mike_getfil('arc_img', subfil=arc_fil, /name, CHKFIL=chkf)
  if CHKF NE 0 then begin
      print, 'mike_mkaimg: File exists: ', out_fil
      if not keyword_set( CLOBBER ) then begin
          print, 'mike_mkaimg: Use /CLOBBER to overwrite.  Skipping...'
          return, out_fil
      endif else print, 'mike_mkaimg: Clobbering -- ', out_fil
  endif

  ;; Read in order structure
  ordr_str = mike_getfil('ordr_str', setup, SIDE=side)
  nordr = n_elements(ordr_str)

  ;; Grab Arc 2D Fit
  arc_2dfit = mike_getfil('arc_2Dfit', subfil=arc_fil)

  ;; Check arc_fil for size
  head = xheadfits(arc_fil)
  sz = lonarr(2)
  sz[0] = sxpar(head, 'NAXIS1')
  sz[1] = sxpar(head, 'NAXIS2')
  ximage = lindgen(sz[0]) # replicate(1,sz[1])
  yimage = lindgen(sz[1]) ## replicate(1,sz[0])

  ;; Grab Slope fit
  mkaimg_str = mike_getfil('arc_fittrc', subfil=arc_fil)

  ;; Create Final image
  aimg = dblarr(sz[0],sz[1])

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; LOOPING

  ;; Offset the order edges by xoff
  ordr_shift = ordr_str
  shft = mike_shifti(shftprm, OSTR=ordr_shift)

  ;; Create the ordermask image
  ordermask = mike_ordermask(sz[0], sz[1], ordr_shift, trim=0.1)

  print, 'mike_mkaimg: Looping on Orders'
  bad_anly = 0 
  for q=0L,nordr-1 do begin
      print, 'mike_mkaimg: Order', ordr_str[q].order
      
      ;; Set order
      mkaimg_ordr = ordr_str[q].order
      
      ;; xcen
      xcen = (ordr_shift[q].lhedg + ordr_shift[q].rhedg)/2.
      
      inorder = where(ordermask EQ ordr_str[q].order, nin)
      if nin GT 250000. then stop
      
      ystart = 1.0d*yimage[inorder]
      xstart = (ximage - xcen ## replicate(1,sz[0]))[inorder]
      ycol = dindgen(sz[1])
      slope_spline = spl_init(ycol, double(ordr_str[q].arc_m))
      
;
;     Four iterations is plenty, converges quickly after the second call 
;
      ywave = ystart
      for islope=1,4 do begin
          slope = spl_interp(ycol, ordr_str[q].arc_m, slope_spline, ywave)
          ywave = ystart - slope*xstart 
          oldslope = slope
      endfor
              
      p = 2.0d * (ywave - arc_2dfit.nrm[0])/arc_2dfit.nrm[1]
      t = (2.0d *(ordr_str[q].order - arc_2dfit.nrmt[0]) $
           /arc_2dfit.nrmt[1])
      worky = flegendre(p, arc_2dfit.ny)
      workt = flegendre(t[0], arc_2dfit.no)
      aimg[inorder] =  $
        worky # transpose(reform(arc_2dfit.res, arc_2dfit.no, $
                                 arc_2dfit.ny) ## workt)

      ys = sort(ywave)
      multi_forw = total( aimg[inorder[ys[1:*]]] -  aimg[inorder[ys]] GT 0)
      multi_back = total( aimg[inorder[ys[1:*]]] -  aimg[inorder[ys]] LT 0)
      if keyword_set(chk) then print, 'mike_mkaimg: mono?  ', multi_forw, multi_back
      if (multi_forw GT 0) AND (multi_back GT 0) then  begin
         print, 'mike_mkaimg: This order is not single-valued in wavelength.'
         print, 'The 2D wavelength fit is likely bad!'
         print, 'Setting mike.flg_anly to 0'
         bad_anly = 1
      endif 
 
  endfor

  if keyword_set( CHK ) then begin
      xatv, aimg, /block, min=3.5, max=4.0
  endif

  if bad_anly GT 0 then begin
    print, 'Bad wavelength Solution, not writing to disk'
    return, out_fil
  endif

  ;; Output
  print, 'mike_mkaimg_work: Writing: ', out_fil
  mwrfits, aimg, out_fil, /create
  print, 'mike_mkaimg_work: Compressing..'
  spawn, 'gzip -f '+out_fil
  
  print, 'mike_mkaimg_work: All done..'
          
  return, out_fil
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro mike_mkaimg, mike, setup, obj_id, side, CHK=chk, CLOBBER=clobber

;

  if  N_params() LT 3  then begin 
      print,'Syntax - ' + $
        'mike_mkaimg, mike, setup, obj_id, [side], /CHK, /CLOBBER [v1.0]'
      return
  endif 

;  Optional Keywords
  if not keyword_set( SIDE ) then side = [1L,2L]

; Loop on side
  for kk=0L,n_elements(side)-1 do begin
      qq = side[kk]
      ;; SIDE
      if qq EQ 1 then print, 'mike_mkaimg: Making BLUE Arcimages' $
      else print, 'mike_mkaimg: Making RED Arcimages'

      ;; Grab all obj indices
      indx = where(mike.flg_anly NE 0 AND mike.side EQ qq AND $
                   mike.obj_id EQ obj_id AND mike.setup EQ setup AND $
                   (strtrim(mike.type,2) EQ 'OBJ' OR $
                   strtrim(mike.type,2) EQ 'STD'), nindx)
      if nindx EQ 0 then begin
          print, 'mike_mkaimg: No Obj found!  Returning' 
          continue
      endif

      arcfil_all= mike[indx].arc_fil
      asort = sort(arcfil_all)
      auniq = uniq(arcfil_all[asort])

      arc_roots = strmid(mike.arc_fil, max(rstrpos(mike.arc_fil, '.')-5), 5)

      ;; LOOP
      for mm=0L,n_elements(auniq)-1 do begin

          arc_fil = arcfil_all[asort[auniq[mm]]]
          idx = indx[asort[auniq[mm]]]
;          xoff = mike[idx].arc_xyoff[0]

          stop ;; Odds are that arc_xyoff is not set!
          rslt = mike_mkaimg_work(arc_fil, setup, qq,$
                                  CHK=chk, CLOBBER=clobber, $
                                  SHFTPRM=mike[idx].arc_xyoff, $
                                  BAD_ANLY=bad_anly) 

          if bad_anly GT 0 then begin
             remove = where(strpos(mike.img_root, arc_roots[idx]) GT -1)
             if remove[0] NE -1 then mike[remove].flg_anly = 0
          endif

          if size(rslt,/tname) NE 'STRING' then stop
      endfor
  endfor

  print, 'mike_mkaimg: All done!'

  return
end
