;+ 
; NAME:
; mike_edgeflat
;     Version 1.1
;
; PURPOSE:
;  To trace the order edges of the individual orders.  And the results are
;  fit (which replaces mike_fittflat).  The fit is a low order fit to the slit
;  width, and then a higher order fit to the order centers.
;  1.  Identify the order edges (interactive is recommended).
;  2.  Performs an order by order tracing of the order edges using
;      trace_crude.
;  3.  Perform (iteratively) a PCA analysis on the coefficients of the
;      individual traces.
;  4.  Create and write to disk a structure summarizing the fits.
;  5.  Do a 2-part fit that replaces the fittflat call.
;
;
; CALLING SEQUENCE:
;   
;  mike_edgeflat, mike, setup, [side], /CHK, 
;
; INPUTS:
;   mike     -  MIKE structure
;   setup    -  Setup identifier 
;   [side]   -  Blue (1), Red (2), or both [1,2L]    (Default: [1,2L])
;
; RETURNS:
;
; OUTPUTS:
;  A structure containing the information for the order by order
;  fits.  This structure is then fed to mike_fittflat to create a 2D
;  solution.  Typical name:  'Flats/TStr_B_01.fits'
;
; OPTIONAL KEYWORDS:
;  /CHK -- Check the order edges interactively (similar to INTER)
;  /INTER -- Determine the order edges interactively
;  SMSHROW -- Row where order edges are determined (default: 1/2 way
;             up the image)
;  THRESH  -- Threshold for an order edge on the red side
;              (default: 100.)
;  /CLOBBER -- Overwrite the output structure
;  P_NSIG  --  Number of sigma significance that an order edge should
;             have for the red side (default: 50.)
;  NSIG  --  Number of sigma significance that an order edge should
;             have fo the blue side (default: 2.0)
;  MINFLAT -- Mininum counts in flat on blue side to use orders (100)
;
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   mike_edgeflat, mike, 1, 1, /clobber
;
; PROCEDURES/FUNCTIONS CALLED:
;   mike_getfil
;   mike_trcflat_edges
;
; REVISION HISTORY:
;   1-Nov-2004  Written by SB, replaces mike_trcflat and mike_fittflat
;                QA file is still in mike_trcflat
;-
;------------------------------------------------------------------------------

pro mike_trcflat_edges, smsh, side, cbin, ledge, redge, CHK=chk, THIN=thin

  ;; Slit width
  swdth = 20.5 * 2. / cbin

  ;; Npix
  npix = n_elements(smsh)

  ;; Auto Thin
  if cbin GT 4 then thin = 1

  ;; Bin ratio
  rt_bin = 2./float(cbin)

  ;; Read in info
  if side EQ 1 then fil_sep = getenv('MIKE_DIR')+'/pro/Flat/blue_sep2x2.idl' $
  else begin
      fil_sep = getenv('MIKE_DIR')+'/pro/Flat/red_sep2x2B.idl' 
  endelse
  restore, fil_sep

  ;; Rebin slit_profile
  nslit = n_elements(slit_prof)
  slit_prof = congrid(slit_prof, round(nslit*2./cbin))

  ;; Convolve
  conv_saw = convol(smsh, slit_prof)

  ;; Find peaks
  srt = sort(conv_saw)
  thresh = 0.2 * conv_saw[ srt[npix-5] ]  ;; -5 deals with Cosmic rays
  dumi = lindgen(npix)
  ap = where(conv_saw GT thresh AND $
             dumi GT 3 AND dumi LT (npix-3))

  if not keyword_set( THIN ) then begin
      ppk = where( conv_saw[ap] GT conv_saw[ap+1] AND $
                  conv_saw[ap] GT conv_saw[ap+2] AND $
                  conv_saw[ap] GT conv_saw[ap-1] AND $
                  conv_saw[ap] GT conv_saw[ap-2], npk)
  endif else begin
      ppk = where( conv_saw[ap] GT conv_saw[ap+1] AND $
                  conv_saw[ap] GT conv_saw[ap-1], npk)
  endelse
  peak = ap[ppk]

;  x_splot, conv_saw, xtwo=peak, ytwo=conv_saw[peak], /block, psym2=1
; peak = [peak, 450., 728., 844.]
; peak = peak[sort(peak)]

  npk = n_elements(peak)

  ;; Fit the separation
  stp = peak - shift(peak,1)
  fitstr = x_setfitstrct(MAXREJ=5, NORD=1, /FLGREJ, LSIG=2., HSIG=2., NITER=3)
  fit = x_fitrej(peak[1:*], stp[1:*], FITSTR=fitstr, REJPT=rejpt)

  ;; Start on 5th peak (or 7th if 5th rejected)
  rejpt = rejpt + 1L
  a = where(rejpt EQ 5L, na)
  if na NE 0 then begin
      a = where(rejpt EQ 7L, na)
      if na NE 0 then stop else istrt = 7L
  endif else istrt = 5L
  
  ;; Head left
  flg_f = 1
  fin_pk = [peak[istrt]]
  dx = x_calcfit(fin_pk[0], FITSTR=fitstr)
  expx = fin_pk[0] - dx
  while expx GT 2. do begin
      ;; Set tolerance
      if flg_f EQ 1 then begin
          toler = 5. 
          flg_f = 0
      endif else toler = 3.
      ;; Look for peak
      mtch = where( abs(expx-peak) LT toler, nmtch)
      case nmtch of
          0: begin  ;; Create a peak
              i0 = (round(expx)-1.5) > 0L
              i1 = (round(expx)+1.5) < (npix-1)
              mx = max( conv_saw[i0:i1], imx)
              fin_pk = [float(imx+i0), fin_pk]
          end
          1: fin_pk = [peak[mtch], fin_pk]
          2: begin
              mx = max( conv_saw[round(peak[mtch])], imx)
              fin_pk = [peak[mtch[imx]], fin_pk]
          end
      endcase
      ;; New expected
      dx = x_calcfit(fin_pk[0], FITSTR=fitstr)
      expx = fin_pk[0] - dx
  endwhile

  ;; Head right
  flg_f = 1
  nfin = n_elements(fin_pk)
  dx = x_calcfit(fin_pk[nfin-1], FITSTR=fitstr)
  expx = fin_pk[nfin-1] + dx
  while expx LT (npix-3.) do begin
      ;; Set tolerance
      if flg_f EQ 1 then begin
          toler = 5. 
          flg_f = 0
      endif else toler = 3.
      ;; Look for peak
      mtch = where( abs(expx-peak) LT toler, nmtch)
      case nmtch of
          0: begin  ;; Create a peak
              i0 = (round(expx)-1.5) > 0L
              i1 = (round(expx)+1.5) < (npix-1)
              mx = max( conv_saw[i0:i1], imx)
              fin_pk = [fin_pk,float(imx+i0)]
          end
          1: fin_pk = [fin_pk,peak[mtch]]
          2: begin
              mx = max( conv_saw[round(peak[mtch])], imx)
              fin_pk = [fin_pk,peak[mtch[imx]]]
          end
      endcase
      ;; New expected
      nfin = nfin + 1
      dx = x_calcfit(fin_pk[nfin-1], FITSTR=fitstr)
      expx = fin_pk[nfin-1] + dx
  endwhile

  if keyword_set( CHK ) then begin
      x_splot, conv_saw, xtwo=fin_pk, ytwo=conv_saw[round(fin_pk)], $
        /block, psym2=1
  endif

  ;; Analyse slit_prof
  nprof= n_elements(slit_prof)
  cen_sprof = float(nprof-1)/2.
  xd = findgen(nprof)
  mx = max(slit_prof,imx)
  i0 = (imx-3) > 0
  i1 = (imx+3) < (nprof-1)
  ledg_sprof = x_centspln(xd[i0:i1], slit_prof[i0:i1], /force)
  mn = min(slit_prof,imn)
  i0 = (imn-3) > 0
  i1 = (imn+3) < (nprof-1)
  redg_sprof = x_centspln(xd[i0:i1], -1.*slit_prof[i0:i1], /force)
  dl = cen_sprof - ledg_sprof
  dr = redg_sprof - cen_sprof

  ;; Grab left edges
  tmp_le = fin_pk - dl
  if tmp_le[0] LT 2. then tmp_le = tmp_le[1:*]
  ledge = fltarr(n_elements(tmp_le))
  for jj=0L,n_elements(tmp_le)-1 do begin
      i0 = round(tmp_le[jj]-3) > 0
      i1 = round(tmp_le[jj]+3) < (npix-2)
      ledge[jj] = i0 + x_centspln(findgen(i1-i0+1), smsh[i0:i1]>0., /force)
  endfor

  ;; Grab right edges
  tmp_re = fin_pk + dr
  
  if tmp_re[nfin-1] GT npix-2. then tmp_re = tmp_re[0:nfin-2]
  redge = fltarr(n_elements(tmp_re))
  for jj=0L,n_elements(tmp_re)-1 do begin
      i0 = round(tmp_re[jj]-3) > 0
      i1 = round(tmp_re[jj]+3) < (npix-2)
      redge[jj] = i0 + x_centspln(findgen(i1-i0+1), (-1.*smsh[i0:i1])>0., /force)
  endfor

  ;; Try to pick up LHS if possible
  l0 = redge[0] - swdth
  if l0 GT 0. AND abs(ledge[0]-l0) GT 5. then begin
      i0 = (round(l0)-1.5) > 0L
      i1 = (round(l0)+1.5) < (npix-1)
      mx = max( smsh[i0:i1], imx)
      ledge = [float(imx + i0), ledge]
  endif

  ;; Final check
  if keyword_set( CHK ) then begin
      x_splot, smsh, xtwo=ledge, ytwo=smsh[round(ledge)], $
        /block, psym2=1, xthr=redge, ythr=smsh[round(redge)], psym3=2
  endif

  return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function mike_trc2d, xcen, mask 

   if NOT keyword_set(sigrej) then sigrej=4.

   ; xcen and xerr come from trace_crude

   ntrace = (size(xcen))[2]
   npix   = (size(xcen))[1]

   good = where(mask,ngood)

   y = (2.*(lindgen(npix,ntrace) mod npix) - npix) / npix 

   nrm0 = 1.0*(ntrace-1)
   t = 2.*(lindgen(npix,ntrace) / npix)/nrm0 - 1  

   work2d = dblarr(npix*ntrace,13)
   worky = flegendre(y[*], 5)
   workt = flegendre(t[*], 5)

   work2d[*,0:4] = workt[*,0:4]
   work2d[*,5:8] = worky[*,1:4] 
   work2d[*,9:12] = worky[*,1:4] * (workt[*,1] # replicate(1,4))

   work2dgood = work2d[good,*]
   alpha = transpose(work2dgood) # work2dgood
   beta = transpose(work2dgood) # (xcen[good])
   choldc, alpha, p, /double
   res = cholsol(alpha,p,beta)
   surffit = xcen * 0.0
   surffit[*] = work2d # res
 
return, surffit
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
pro mike_edgeflat, mike, setup, side, CHK=chk, SMSHROW=smshrow, $
                  CLOBBER=clobber, P_NSIG=p_nsig, THRESH=thresh, DEBUG=debug, $
                  INTER=inter, NSIG=nsig, MINFLAT=minflat, NOBASIS=nobasis, $
                  NOSTOP=nostop

;
  if  N_params() LT 2  then begin 
      print,'Syntax - ' + $
        'mike_edgeflat, mike, setup, [side], P_NSIG=, THRESH=, /CLOBBER,'
      print, '           /CHK, /INTER, /DEBUG, NSIG=, MINFLAT= [v1.1]'
      return
  endif 

  ;; Optional Keywords
  if not keyword_set( SIDE ) then side = [1L,2L]
  if not keyword_set( P_NSIG) then p_nsig = 50.
  if not keyword_set( NSIG) then nsig = 5.0
  if not keyword_set( THRESH) then thresh = 100.
  if not keyword_set( BGUESS) then BGUESS = 75L
  if not keyword_set( NCOEFF) then ncoeff = 6L
  if not keyword_set( MINFLAT) then minflat = 100.


  x_psclose
  !p.multi=[0,1,1]

; Loop on side

  for iside=0L,n_elements(side)-1 do begin
      qq = side[iside]
      ;; SIDE
      if qq EQ 1 then print, 'mike_edgeflat: Tracing BLUE trace flat' $
      else print, 'mike_edgeflat: Tracing RED trace flat'

      ;; Check output file
      trc_fil = mike_getfil('tflat_str', setup, SIDE=qq, /name, CHKFIL=chkf)
      if CHKF NE 0 AND not keyword_set( CLOBBER ) then begin
          print, 'mike_edgeflat: Trace exists, moving on..'
          continue
      endif

      ;; Read in flat 
      flat_file = mike_getfil('tflat_fil', setup, SIDE=qq, /name)
      flat = xmrdfits(flat_file)
      flativar = xmrdfits(flat_file,1)
      flativar = flativar*(smooth(1.0*(flativar EQ 0),3) EQ 0)
      sz = size(flat, /dimensions)

      ;; Set binning
      cbin = round(2048. / sz[0])
      rbin = round(4096. / sz[1])
      
      ntmp = 0L
  
      ;; Sawtooth
      sf =  shift(flat ,1) 
      sb =  shift(flat,-1) 

      ;; the denominator was biasing the centroid with this earlier definition:
      ;;    saw =  (sf-sb)/(abs(sf+sb) - 2.0*min(flat) + 10)
      ;; 
      ;; Let's normalize with this function
      ;; sawnorm = djs_median(flat, 1) ## replicate(1,sz[0])
      ;; saw =  (sf - sb)/(sawnorm + (sawnorm EQ 0)) * (sawnorm GT 0)
      ;;
      ;; Let's just subtract the two, otherwise we might bias the centroid

      ;; ipos = 1 are the LHS of each order; ipos=2 are the RHS

      saw =  -1.*(sf - flat)
      sawivar = flativar/2.0

      ;; Smash
      if not keyword_set( smshrow ) then smshrow = sz[1]/2 else begin
          if abs(smshrow - sz[1]/2) GT 100 THEN begin
              print, 'mike_edgeflat:  Continue at your own risk!'
              stop
          endif
      endelse
      print, 'mike_edgeflat: Using row ',strtrim(smshrow,2), $
        ' to identify orders'
      smsh = djs_median(saw[*,smshrow-15:smshrow+15],2)

      ;; Smooth
      nfilter = long(sz[0]/20.)
      smsh_denom = djs_median(convol(flat[*,smshrow-15:smshrow+15],  $
                                     replicate(1./nfilter,nfilter), $
                                     /edge_truncate) + 1,2) 
      smsh[0] = 0
      smsh[sz[0]-1]= 0
      smsh_search = smooth((smsh/(smsh_denom > 1)),3) 

      mike_trcflat_edges, smsh_search, qq, cbin, ledge, redge, CHK=chk

      ledge = ledge[sort(redge)]
      redge = redge[sort(redge)]

      npk = n_elements(redge)
      if redge[0] LT ledge[0] then begin
        width = (redge[1:*] - ledge)
        width_p = ladfit(findgen(npk-1),width)

        ledge = [redge[0]-width_p[0] + width_p[1], ledge[0:npk-2]]
      endif

          
      ystart = replicate(smshrow, npk)
      lsaw = saw > 0. 
      lsaw[0,*] = 0
      lsaw[sz[0]-1,*] = 0
          
      rsaw = (-1.*saw) > 0.
      rsaw[0,*] = 0
      rsaw[sz[0]-1,*] = 0
         
      lstart = ledge
      rstart = redge
      for j=0L,9 do begin & $
        lstart = trace_fweight(smsh>0, lstart, ystart*0, radius=2.) & $
        rstart = trace_fweight((-1.0*smsh)>0,rstart,ystart*0, radius=2.) & $
      endfor
             
      lcen = trace_crude(lsaw, yset=ly,XSTART=lstart,$
                             radius=3.5, ystart=smshrow, xerr=lerr , $
                             maxshifte=0.2)
      l2cen = mike_fweight(lsaw, lcen, ly, invvar=sawivar, $
                               radius=2, xerr=l2err,sig=0.2) 

      rcen = trace_crude(rsaw, yset=ry,XSTART=rstart,$
                             radius=3.5, ystart=smshrow, xerr=rerr , $
                             maxshifte=0.2)
      r2cen = mike_fweight(rsaw, rcen, ry, invvar=sawivar, $
                               radius=2, xerr=r2err,sig=0.2) 
          

       l2ivar = (l2err LT 1.0 AND l2cen GT 10)/l2err^2
       r2ivar = (r2err LT 1.0 AND r2cen GT 10)/r2err^2
         

       y = [[ly],[ry]]
       x2cen = [[lcen],[rcen]]
       x2cen_ivar = [[l2ivar],[r2ivar]]

       xy2traceset, y, x2cen, tset, invvar=x2cen_ivar, maxdev=1.0, maxrej=1,$
             /groupbadpix, yfit=x2fit, ncoeff=ncoeff, outmask=outmask2, /silent
        
       nrow = (size(y))[1] 
       msktrc = where((total(outmask2[*,0:npk-1],1) GT nrow*0.7) * $
                   (total(outmask2[*,npk:*],1) GT nrow*0.7) EQ 0)
                
       msktrc = [msktrc, msktrc+npk]
 
       ;; PCA
       x3fit = mike_basis(tset, x2cen, outmask2, $
                               ncoeff=ncoeff,eigenvec=eigenvec, poly_n=3, $
                               msktrc=msktrc, outstr=pca_out, /skipx0) 

       xy2traceset, findgen(npk)# replicate(1,2), reform(pca_out.x0,npk,2),$
                  x0set, maxdev=0.3, maxrej=1,ncoeff=4,yfit=x0fit,outmask=x0msk

       x3fit = mike_basis(tset, x2cen, outmask2, $
                               ncoeff=ncoeff,eigenvec=eigenvec, poly_n=3, $
                               msktrc=msktrc, outstr=pca_out, x0=x0fit[*]) 



       ;; Recentroid
       l4cen = x3fit[*,0:npk-1] 
       r4cen = x3fit[*,npk:*] 
          
       for j=1L,7 do begin & $
         l4cen = trace_gweight(lsaw,l4cen,ly,sigma=1,xerr=l4err,inv=sawivar) & $
         r4cen = trace_gweight(rsaw,r4cen,ry,sigma=1,xerr=r4err,inv=sawivar) & $
       endfor
          
       diff4 = (r4cen - l4cen) 
       diff4err = sqrt(l4err^2 + r4err^2) + 900.0*(l4err EQ 0 OR r4err EQ 0)
       t= 0
       trace2d, diff4, diff4err, diff_fit, nycoeff=4, ntcoeff=3, $
                res=diffres, t=t, mask=diffmask

       avg4 = 0.5*(r4cen + l4cen)
       nycoeff = 6
       trace2d, avg4, diff4err, avgfit, nycoeff=nycoeff, ntcoeff=4, $
                res=avgres, mask=avgmask

       x4cen = [[l4cen],[r4cen]]
       x4fit = [[avgfit-0.5*diff_fit], [avgfit+0.5*diff_fit]]
          ;; Save for QA
       sv_qa1 = { $
                  pca_out: pca_out, $
                  x4cen: x4cen, $
                  x4fit: x4fit, $
                  ycen: y,  $
                  npk: npk,  $
                  msk: outmask2 $
                }

          
       nneg = npk
       npos = npk
       tmp_xcen = x4cen - 0.5
       tmp_xerr=  [[l4err],[r4err]]
       tmp_xfit = x4fit - 0.5
       ntmp = npos+nneg
    
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ;; QA
      qafil = mike_getfil('qa_trcflat', setup, SIDE=qq)
      print, 'mike_edgeflat: QA file is '+qafil
      x_psopen, qafil, /maxs
      clr = getcolor(/load)

      if keyword_set(sv_qa1.pca_out) then begin
          !p.multi=[0,ncoeff/2 + (ncoeff mod 2),2,0,1]

            in = lindgen(2*npk) mod npk
            bad = where(sv_qa1.pca_out.x0msk EQ 0B, nbad)
            plot, in, sv_qa1.pca_out.x0, psym=1, $
              color=clr.black, background=clr.white, charsize=1.5, /yno
            if nbad NE 0 then oplot, [in[bad]], [sv_qa1.pca_out.x0[bad]], $
              psym=2, color=clr.red
            oplot, sv_qa1.pca_out.x0fit[0:npk-1], color=clr.blue
            oplot, sv_qa1.pca_out.x0fit[npk:*], color=clr.blue
            mn = min(sv_qa1.pca_out.x0fit, max=mx)
            xyouts, 1., mn + (mx-mn)*0.9, 'x0', charsize=1.5

            ;; PCA
            for ii=0L,ncoeff-2 do begin
              plot, sv_qa1.pca_out.usetrc mod npk,sv_qa1.pca_out.hidden[ii,*], $
                 color=clr.black, background=clr.white, $
                 xrange=[0L,sv_qa1.npk-1], psym=1, charsize=1.5, /yno
              oplot, sv_qa1.pca_out.high_fit[0:npk-1,ii], color=clr.blue
              oplot, sv_qa1.pca_out.high_fit[npk:*,ii], color=clr.blue
              ;; Rej points
              case ii of 
                  0L: begin
                      if sv_qa1.pca_out.rejpt.rej0[0] NE -1 then $
                        oplot, [sv_qa1.pca_out.usetrc[sv_qa1.pca_out.rejpt.rej0]] mod npk, $
                        [sv_qa1.pca_out.hidden[ii,sv_qa1.pca_out.rejpt.rej0]], $
                        color=clr.red, psym=2
                  end
                  1L: begin
                      if sv_qa1.pca_out.rejpt.rej1[0] NE -1 then $
                        oplot, [sv_qa1.pca_out.usetrc[sv_qa1.pca_out.rejpt.rej1]] mod npk, $
                        [sv_qa1.pca_out.hidden[ii,sv_qa1.pca_out.rejpt.rej1]], $
                        color=clr.red, psym=2
                  end
                  else: 
              endcase
          ;; Label
              mn = min(sv_qa1.pca_out.hidden[ii,*], max=mx)
              xyouts, 1., mn + (mx-mn)*0.9, 'PCA'+strtrim(ii,2), charsize=1.5
          endfor


       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ;; Traces 
       !p.multi=[0,1,1]
       tt = tmp_xfit+0.5  ;; Offset
       gd = where(sv_qa1.msk EQ 1B, complement=bad)
        plot, sv_qa1.ycen[gd], 10*(sv_qa1.x4cen[gd]-tt[gd])+tt[gd], $
          psym=3, color=clr.black, background=clr.white, charsize=1.5, $
          xrange=[0., sz[1]], yrange=[0.,sz[0]], /xs, /ys
        !p.thick=1
       if bad[0] NE -1 then $
          oplot, sv_qa1.ycen[bad], 10*(sv_qa1.x4cen[bad]-tt[bad])+tt[bad], $
            psym=3, color=clr.red
       for ii=0L,npk-1 do begin
          oplot, findgen(sz[1]), tt[*,ii], color=clr.green
          oplot, findgen(sz[1]), tt[*,ii+npk], color=clr.blue
       endfor
       xyouts, 0.5, 0.96, $
        'Reduced chi^2 = '+string(sv_qa1.pca_out.red_chi2,format='(f7.4)'), $
         color=clr.black, /normal, charsize=2.5, alignment=0.5
      endif

      x_psclose
      !p.multi=[0,1,1]
      spawn, 'gzip -f '+qafil

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ACCOUNTING

      start_guess_order = qq EQ 2 ? 80 : BGUESS
      
      swidth = median(diff_fit)

      print, 'mike_edgeflat: Adopting order width of ', swidth, ' binned pixels'

      ;; Fill up the output
      trc_str = { $
                  xcen: tmp_xcen,$
                  xerr: tmp_xerr,$
                  xfit: tmp_xfit,$
                  flg: intarr(ntmp), $
                  nordr: npk, $
                  smshrow: smshrow, $
                  swidth: swidth, $
                  orderstrt: start_guess_order $
                }

      ;; Zero out bad few columns of red side:: 
      ;; Is this setup dependent?!  (i.e. the dichroic)
      ;; If so, it will need to be a function of order #
      
      if qq EQ 2 then begin
          bd = where(trc_str.xcen LT 10., nbd)
          if nbd NE 0 then trc_str.xerr[bd] = 99.
      endif

      ;; Fill up flg
      trc_str.flg[0:nneg-1] = 1 ; LHS of slit
      trc_str.flg[nneg:ntmp-1] = 2 ; RHS of slit
      
      ;; Write 
      mwrfits, trc_str, trc_fil, /create


;
;     Now write out the tflat fit
;
      tmp = { $
               order: 0L, $     ;  Physical order #
               flg_anly: 0, $   ;  1=ok, 0=ng
               xcen: 0., $      ;  Center of order at ycen
               ycen: 0., $      ;  Smash row
               ymin: 0L, $      ;  Minimum used in y direction (-> -1)
               ymax: nrow, $    ;  Maximum used in y direction (-> +1)
               lcen: 0.d, $     ;  Wavelength correspondoning to xcen,ycen
               lhedg: fltarr(nrow), $ ; LH edge
               rhedg: fltarr(nrow),  $ ; RH edge
               arc_m: fltarr(nrow), $
               profile0 : fltarr(251),$ ; Slit profile
               profile1 : fltarr(251),$ ; Slit profile
               lhc : fltarr(nycoeff),$ ; LH coefficients
               rhc : fltarr(nycoeff) $ ; RH coefficients
             }

       smrow = trc_str.smshrow

       ord_str = replicate(tmp, npk)
       ord_str.ycen = float(smrow)

       ;; Fill in edges
       ord_str.lhedg = trc_str.xfit[*,0:npk-1]
       ord_str.rhedg = trc_str.xfit[*,npk:*]
       ord_str.xcen = (ord_str.lhedg[smrow]+ord_str.rhedg[smrow,*])/2.

       tbasis = transpose(t[0,*])

       avgbasis = transpose(reform(avgres,4,nycoeff) ## flegendre(tbasis, 4))
       diffbasis = transpose(reform(diffres,3,4) ## flegendre(tbasis, 3))

       ord_str.lhc = avgbasis
       ord_str.rhc = avgbasis

       ord_str.lhc[0:3] = ord_str.lhc[0:3] - 0.5*diffbasis
       ord_str.rhc[0:3] = ord_str.rhc[0:3] + 0.5*diffbasis

       ord_str.lhc[0] = ord_str.lhc[0] - 0.5
       ord_str.rhc[0] = ord_str.rhc[0] - 0.5
       

       ord_str.order = lindgen(n_elements(ord_str)) + 1000

       ordr_fil = mike_getfil('ordr_str', setup, SIDE=qq, CHKFIL=chkf, /name)
       if CHKF NE 0 AND $
         not keyword_set( CLOBBER ) then begin
           print, 'mike_edgeflat: Order structure exists. Moving on!', ordr_fil
           continue
       endif

       ;; Write to fits
       print, 'mike_edgeflat: Writing order structure ', ordr_fil
       mwrfits, ord_str, ordr_fil, /create

  endfor
  ;; 
  print, 'mike_edgeflat: You may now wish to check the results with mike_chktrcflat'
  print, 'mike_edgeflat: All done!'

  return
end
