;+ 
; NAME:
; mike_gain
;     Version 1.1
;
; PURPOSE:
;    Calculate the inverse gain, e-/DN with two very similiar input images
;      For MIKE, milky flats or well illuminated flats are best.
;
; CALLING SEQUENCE:
;   
;  estimated_gain = mike_gain(image1, image2)
;
; INPUTS:
;   image1  -  Any array of pixels with reasonable gaussian errors
;   image2  -  A very similar image to image1, but not identical!
;
; RETURNS:
;  gain   -  A floating point number: the inverse gain
;
; OUTPUTS:
;
; OPTIONAL KEYWORDS:
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;   The gain is calculated from the variance of the natural logarithm 
;     of the ratio of two images which have units of ADU (Poisson counts / gain)
;
;     <gain> = 1.0 / { variance [ sqrt(reduced_flux) * ln(i1/i2) ] } 
;
; EXAMPLES:
;      image1 = readfits('ov_mr0405.fits') 
;      image2 = readfits('ov_mr0406.fits')
;      gain = mike_gain(image1, image2)
;
; PROCEDURES/FUNCTIONS CALLED:
;
; REVISION HISTORY:
;   16-Jul-2003 Written by SMB
;-
;------------------------------------------------------------------------------

function mike_gain, image1, image2

  if  N_params() LT 2  then begin 
      print,'Syntax - ' + $
        'mike_gain(image1, image2) (v1.1)'
      return, -1
  endif 
  
     ; the >1 trick forces the minimum value to be GE to 1.
     ;  so no divide by zero worries

    i1 = transpose(image1)
    i2 = transpose(image2)

    pos = where(i1 GT 100 AND i2 GT 100, npos)
    if npos LT 1000 then begin
        print, 'Not enoung positive pixel pairs'
        return, 0.0
    endif
   
     x =  i1[pos]/i2[pos]
     r = median(x)
       print, 'Ratio of images is ', r
     reduced_flux = 1.0 / (1./i1[pos]  + 1./i2[pos])

     guess = sqrt(reduced_flux)* alog(x/r)
     sig_guess = stddev(guess)
     keep = where(abs(guess) LT 4.0*sig_guess AND $
                  reduced_flux GT median(reduced_flux) , nkeep)

     if nkeep LT 100 then begin
        print, 'Too many outliers'
        return, 0.0
     endif


     smooth_guess = smooth(guess[keep],11) 

     gain = 1./(mean((guess[keep]-smooth_guess)^2)*11./10.)


     ;  gain_array = reduced_flux * (x/r-1)^2  
     ; element which is at the border of 1-sigma 
     ;  sigma_1 = n_elements(pos) * errorf(1./sqrt(2.0))
     ; a quicker way than sort to find the approx position
     ; only good to about 0.0001
     ; gain_hist = histogram(gain_array, bins=0.0001, min=0, max=15)
     ; gain = min(where(total(gain_hist,/cumul) GT sigma_1)) * 0.0001

     ;; what about difference 

     ; sm1 = transpose(median_row(transpose(image1),11))
     ; sm1_med = median(sm1[where(sm1 GT 100)])
     ; pos1 = where(image1 GT 0 AND sm1 GT sm1_med*0.5 AND sm1 LT sm1_med*2)
     ; diffgain1 = 0.4543/median((image1[pos1] -sm1[pos1])^2/(sm1[pos1]))

     ; sm2 = transpose(median_row(transpose(image2),11))
     ; sm2_med = median(sm2[where(sm2 GT 100)])
     ; pos2 = where(image2 GT 0 AND sm2 GT sm2_med*0.5 AND sm2 LT sm2_med*2)
     ; diffgain2 = 0.4543/median((image2[pos2] -sm2[pos2])^2/(sm2[pos2])) 
     ;   print, 'Difference gain measurement is ', diffgain1, diffgain2


     return, gain
end

