;+
; NAME:
; mike_1dspec
;    Version 1.1
;
; PURPOSE:
;   Combines orders of a echfspec structure into a 1d spectrum.  
;   Outputs flux and error arrays into separate files (_F.fits, _E.fits)
;
; CALLING SEQUENCE:
;  
;   mike_1dspec, mike, setup, obj_id, side
;
; INPUTS:
;   mike      - MIKE structure
;   obj_id   -  Object ID  (e.g. 0L, 1L, etc)
;   [exp_id] -  Exposure frames (e.g. [0L, 1L])
;
; RETURNS:
;
; OUTPUTS:
;   1d flux      -   (fits file; FSpec/name_ech_F.fits)
;   1d error     -   (fits file; FSpec/name_ech_E.fits)
;
; OPTIONAL KEYWORDS:
;    /SILENT   - No text output
;    OBJ_NM=   - Name of object in slit (a = science)
;    /STD      - Run on a standard star
;    OUTNM=    - Alternative output name for FSpec file
;    ENDTRM=   - Trim order edges by this when calculating ratios
;    MINPIX1=  - Minimum 'good' pixels to calculate fitting profile
;    ORDNM=    - 0th or 1st order fitting (1 = 1st, 2 = 0th (default))
;    SNRMIN=   - Minimum S/N per pixel for consideration in fitting
;    MINPIX2=  - Minimum 'good' pixels to calculate simple 0th order fit
;
; OPTIONAL OUTPUTS:
;
; COMMENTS: ;
; EXAMPLES:
;   mike_1dspec, mike, setup, obj_id, side
;
; PROCEDURES/FUNCTIONS CALLED:
;
; REVISION HISTORY:
;   14-Jun-2004 Written by GEP
;-
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


pro mike_1dspec, mike, setup, obj_id, side, exp_id, SILENT=silent, LIST=list, $
                 OBJ_NM=OBJ_NM, STD=std, OUTNM=outnm, CHK=chk, ORDNM=ordnm, EDGEORDR=edgeordr,$
                 SNRMIN=snrmin, FLUX=flux, KEYINDX=keyindx, MAXVAR=maxvar, LCOLBIN=lcolbin, $
                 NOFLUX=noflux, MINPIX1=minpix1, ENDTRM=endtrm, MINPIX2=minpix2, LSIDE=lside, $
                 MIDORDR=midordr, DOEDGE=doedge

  if N_params() LT 4 and not keyword_set(LIST) then begin 
    print, 'Syntax - ' + $
      'mike_1dspec, mike, setup, obj_id, side, [exp_id], /SILENT ' + $
      'OBJ_NM=, ORDNM=, /CHK '
    print, '     LIST= [v1.0]'
    return
  endif

  if not keyword_set(OBJ_NM) then obj_nm = 'a'
  if not keyword_set(EDGEORDR) then edgeordr = 2L
  if not keyword_set(DOEDGE) then edgeordr=0L
  if not keyword_set(MIDORDR) then midordr = 1L
  if not keyword_set(SNRMIN) then snrmin = 2.
  if not keyword_set(MINPIX1) then minpix1 = 300L
  if not keyword_set(MINPIX2) then minpix2 = 50L
  if not keyword_set(MAXVAR) then maxvar = 0.4
  if keyword_set (LIST) then begin
      if keyword_set (LSIDE) then side = lside else side = 1L
      if not keyword_set (LCOLBIN) then lcolbin = 2L
  endif

  if keyword_set (LIST) then $ 
    print, 'mike_1dspec:  2d --> 1d-ing the 1st file in the following list -- ', list
  
; Set exp
  if not keyword_set( LIST) then begin
    if not keyword_set( STD ) then begin
      allexp = where(mike.type EQ 'OBJ' AND mike.flg_anly NE 0 AND $
                     mike.setup EQ setup AND mike.obj_id EQ obj_id $
                     AND mike.side EQ side)
      if keyword_set(exp_id) then exp = allexp[exp_id] else exp=allexp
      nexp = n_elements(exp)
    endif else begin ;; STD
      exp = obj_id[0]
      nexp = 1
      obj_id = 99L
    endelse 
  endif else begin 
    obj_id = 99L
    readcol, list, files, FORMAT='A'
    nexp = n_elements(files)
    if nexp NE 3 then begin
        print, 'mike_1dspec LIST format requires 3 files to be listed in ' + $
                'list file:  first file is object file to be 1d-ed, second file ' +$
                'is flux output file, third is sigma output file.  Returning.'
        return
    endif
  endelse

;;;;;;;;;;;;;
;; Open file

  if not keyword_set( SILENT ) then $
    print, 'mike_1dspec: Loading up the files....'
    
  if not keyword_set( LIST ) then begin
    if keyword_set ( OUTNM ) then subfil = outnm+obj_nm $
    else subfil = strcompress(strtrim(mike[exp[0]].Obj,2),/remove_all) + obj_nm
    if side EQ 1 then clrc = '_b' else clrc = '_r'
    infil = 'FSpec/'+subfil+clrc+'.fits'
    print, 'mike_1dspec: Reading ', infil
    spec = xmrdfits(infil,1)
    head = xheadfits(infil)
  endif else begin
    spec = xmrdfits(files[0],1)
    head = xheadfits(files[0])
  endelse

;;;;;;;;;;;;
;;; begin 1d

    use = where(spec.phys_ordr NE 0,nuse)

    gd = where(spec.wave NE 0.)
    minwv = min(spec.wave[gd])
    maxwv = max(spec.wave[gd])
 
    if not keyword_set(ORDNM) then ordnm = 2

    if not keyword_set( LIST ) then $
        velpix = (side EQ 1 ? 1.50d : 2.10d) * double(mike[exp[0]].colbin) $
    else velpix = (side EQ 1 ? 1.50d : 2.10d) * double(lcolbin)

    cdelt = alog10(1.0d + velpix/299792.458d)
    npix = 80000L
    tot_wave = 10^(alog10(3000.0d) + dindgen(npix)*cdelt)
    weight = fltarr(npix)
    weight = weight*0.
    tot_flux = fltarr(npix)
    sig = replicate(-1.,npix)

    sxaddpar, head, 'CDELT1', cdelt
    sxaddpar, head, 'CRPIX1', 1
    sxaddpar, head, 'CTYPE1', 'LINEAR'
    sxaddpar, head, 'DC-FLAG', 1
    sxaddpar, head, 'BITPIX', -32
    sxaddpar, head, 'NAXIS', 1
    sxaddpar, head, 'NAXIS1', n_elements(tot_flux)

    medrto = fltarr(nuse)
    medrto[*] = 1.
    truerto = fltarr(nuse)
    truerto[0] = 1.
    
    loop1zero = floor(nuse/2.)
    loop2zero = loop1zero

    ;; Get medrto for each order overlap

    if keyword_set( CHK ) then begin
    for ii=1,nuse-2 do begin
            title = strtrim(string(use[ii]),2)
            u1 = where(spec.wave[*,use[ii]] GT 0)
            u2 = where(spec.wave[*,use[ii-1]] GT 0)
            u3 = where(spec.wave[*,use[ii+1]] GT 0)
            x_splot, spec.wave[u1,use[ii]], spec.fx[u1,use[ii]], ytwo=spec.fx[u2,use[ii-1]], xtwo=spec.wave[u2,use[ii-1]], ythr=spec.fx[u3,use[ii+1]], xthr=spec.wave[u3,use[ii+1]], psym3=-3, /block, TITLE=title
        endfor
    endif

    for ii=0,nuse-1 do begin
        indx = where(abs(tot_wave - spec.wave[0,use[ii]]) LT 0.006, nindx)
        if nindx EQ 0 then stop
        indx = indx[0]

        a = where(spec.var[*,use[ii]] GT 0, ngd)
        if ngd NE 0 then begin
            wtmp = spec.var[a,use[ii]]
            tmp = tot_flux[indx+a]+spec.fx[a,use[ii]] / wtmp
            for i=0L, ngd-1 do begin
                tot_flux[indx+a[i]] = tmp[i]
                weight[indx+a[i]] = weight[indx+a[i]] + 1./wtmp[i]
            endfor
        endif
    endfor
    a = where(weight NE 0., na)
    if na EQ 0 then stop
    tot_flux[a] = tot_flux[a] / weight[a]
    sig[a] = sqrt(1./weight[a])
    
      x_splot, tot_wave[a], tot_flux[a], ytwo=sig[a], /block
    if keyword_set( CHK ) then $
      x_splot, tot_wave[a], tot_flux[a], ytwo=sig[a], /block
    minwv = min(tot_wave[a])


    sxaddpar, head, 'CRVAL1', alog10(minwv)

    if not keyword_set( LIST ) then begin
      if keyword_set ( OUTNM ) then subfil = outnm+obj_nm $
      else subfil = strcompress(strtrim(mike[exp[0]].Obj,2),/remove_all) + obj_nm
      if side EQ 1 then clrc = '_b' else clrc = '_r'
      outfil = 'FSpec/'+subfil+clrc+'_F.fits'
      sigfil = 'FSpec/'+subfil+clrc+'_E.fits'
    endif else begin
      outfil = files[1]
      sigfil = files[2]
    endelse

    if not keyword_set( SILENT ) then $
      print, 'mike_1dspec: Writing results to files: ', outfil, sigfil

    mwrfits, tot_flux[a], outfil, head, /create, /silent
    mwrfits, sig[a], sigfil, head, /create, /silent

end
