;+ 
; NAME:
; mike_combspec
;    Version 1.1
;
; PURPOSE:
;   Combines multiple exposures of the same obj
;    Must be run even on an object with a single exposure
;
; CALLING SEQUENCE:
;   
;   mike_combspec, mike, setup, obj_id, side, exp_id
;
; INPUTS:
;   mike      - MIKE structure
;   obj_id   -  Object ID  (e.g. 0L, 1L, etc)
;   [exp_id] -  Exposure frames (e.g. [0L, 1L])
;
; RETURNS:
;
; OUTPUTS:
;   echfspec      -  ESI fspec structure (fits file; FSpec/name_ech.fits)
;
; OPTIONAL KEYWORDS:
;    /SILENT   - No text output
;    LIST=     - Use an input list to combine (required for multiple
;                night obs).  Output is in 'FSpec/tmp_ech.fits'.
;    OBJ_NM=   - Name of object in slit (a = science)
;    /STD      - Run on a standard star
;    OUTNM=    - Alternative output name for FSpec file
;    /NOFLUX   - Fill echfspec structure with fx, rather than flux array 
;    ENDTRM=   - Trim order edges by this when calculating ratios
;    MINPIX1=  - Minimum 'good' pixels to calculate fitting profile
;    ORDNM=    - Fitting profile order (default: 1)
;    SNRMIN=   - Minimum S/N per pixel for consideration in fitting
;
; OPTIONAL OUTPUTS:
;
; COMMENTS: ;
; EXAMPLES:
;   mike_combspec, mike, setup, obj_id, side
;
; PROCEDURES/FUNCTIONS CALLED:
;
; REVISION HISTORY:
;   04-Jan-2004 Written by JXP
;   10-Jun-2004 Revisited by GEP
;-
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

pro mike_combspec, mike, setup, obj_id, side, exp_id, SILENT=silent, LIST=list, $
                   OBJ_NM=OBJ_NM, STD=std, OUTNM=outnm, SNRMIN=snrmin, $
                   FLUX=flux, KEYINDX=keyindx, MEDINDX=medindx, NOFLUX=noflux, $
                   MINPIX1=minpix1, ENDTRM=endtrm, ORDNM=ordnm, CHK=chk, SIGREJ=sigrej, $
                   LSIDE=lside

;
  if  N_params() LT 4  and not keyword_set(LIST) then begin 
    print,'Syntax - ' + $
      'mike_combspec, mike, setup, obj_id, side, [exp_id], /SILENT, ' + $
      'OBJ_NM=, /PATCH '
    print, '      LIST= [v1.1]'
    return
  endif 
  if keyword_set(LIST) and keyword_set(LSIDE) then side = lside
  if keyword_set(LIST) and not keyword_set(LSIDE) then begin
      print, 'Warning -- with LIST keyword the side needs to be explicitly set,' + $
              ' with use of LSIDE keyword, defaulting to blue!'
      side=1
  endif

;  Optional Keywords
  if not keyword_set(KEYINDX) then keyindx = 0L
  if not keyword_set(MEDINDX) then medindx = 100
  if not keyword_set(ENDTRM) then endtrm = 0
  if not keyword_set(ORDNM) then ordnm = 1
  if not keyword_set(SIGREJ) then sigrej=4.
  if not keyword_set(MINPIX1) then minpix1 = 100
  if not keyword_set(OBJ_NM) then obj_nm = 'a'
  if not keyword_set( CORDR ) then begin
      if side EQ 1 then CORDR = 85L else CORDR=55L
  endif
  if not keyword_set( SORDR ) then begin
      if side EQ 1 then SORDR = 70L else SORDR=30L
  endif
  if not keyword_set( SNRMIN ) then snrmin = 2.

  if keyword_set( LIST ) then $
    print, 'mike_echcombspec: Combining Obj files from this list -- ', list

; Set exp
  if not keyword_set( LIST ) then begin
      if not keyword_set( STD ) then begin
          allexp = where(mike.type EQ 'OBJ' AND mike.flg_anly NE 0 AND $
                         mike.setup EQ setup AND mike.obj_id EQ obj_id $
                        AND mike.side EQ side)
          if keyword_set(exp_id) then exp = allexp[exp_id] else exp=allexp
          nexp = n_elements(exp)
      endif else begin  ;; STD
          exp = obj_id[0]
          nexp = 1
          obj_id = 99L
      endelse
  endif else begin
      obj_id = 99L
      readcol, list, files, FORMAT='A'
      nexp = n_elements(files)
  endelse


;;;;;;;;;;
; Open Files

  if not keyword_set( SILENT ) then $
    print, 'mike_echcombspec: Loading up the files...'

; Open Obj files
  for q=0L,nexp-1 do begin
      ;; Read
      if not keyword_set( LIST ) then begin
          print, 'mike_combspec: Reading ', mike[exp[q]].obj_fil
          if x_chkfil(mike[exp[q]].obj_fil+'*') EQ 0 then begin
              print, 'mike_echcombspec: Obj file doesnt exist! Returning..', $
                mike[exp[q]].obj_fil
              return
          endif
          tmp = xmrdfits(mike[exp[q]].obj_fil, 1, STRUCTYP='mikeobjstrct', $
                         /silent)
      endif else begin
          if x_chkfil(files[q]+'*') EQ 0 then begin
              print, 'mike_echcombspec: Obj file doesnt exist! Returning..', $
                files[q]
              return
          endif
          tmp = xmrdfits(files[q], 1, STRUCTYP='mikeobjstrct', /silent)
      endelse
      ;; Grab good obj
      gdobj = where(tmp.obj_id EQ obj_nm, ngd)
      ;; Add to total structure
      if q EQ 0 then mikeobj = tmp[gdobj] else mikeobj = [mikeobj, tmp[gdobj]]
  endfor

; Setup orders
  all_ordr = mikeobj.order
  all_ordr = all_ordr[uniq(all_ordr,sort(all_ordr))]

  if side EQ 1 then begin ; Blue
      ordrs = [min(all_ordr), max(all_ordr)]
      ostep = 1
  endif else begin ; Red
      ordrs = [min(all_ordr), max(all_ordr)]
      ostep = 1
  endelse

  nspec = nexp

;;; CREATE FINAL 2D ;;;

  if not keyword_set(LIST) then begin
      if keyword_set( OUTNM ) then subfil = outnm+obj_nm $
      else subfil = strcompress(strtrim(mike[exp[0]].Obj,2),/remove_all)+obj_nm
;      outfil = mike_getfil('fspec_fil',subfil=subfil, SIDE=side, /name)
      if side EQ 1 then clrc = '_b' else clrc = '_r'
      outfil = 'FSpec/'+subfil+clrc+'.fits'
  endif else outfil = 'FSpec/tmp_ech.fits' 

  echfspec = { mikefspecstrct }

  ;; Copy
  echfspec.nexp = nspec
  ;; Center ordr
  all_cen = where(mikeobj.order EQ CORDR, ncen)
  if ncen NE nspec then stop

  ;; Set texp
  for i=0L,nspec-1 do echfspec.texp[i] = mikeobj[all_cen[i]].exp

  ;; Other tags
  copy_struct, mikeobj[0], echfspec, EXCEPT=["wave","fx","var","npix"]
  if not keyword_set(LIST) then $
      for i=0L,nspec-1 do echfspec.obj_fil[i] = mike[exp[i]].obj_fil $
  else for i=0L,nspec-1 do echfspec.obj_fil[i] = files[i]

  svmedian = 1.
  svmedian = replicate(svmedian,nspec,ordrs[1]+1)
  svsnr = 1.
  svsnr = replicate(svsnr,nspec,ordrs[1]+1)
  ordr_typ = -1
  ordr_typ = replicate(ordr_typ,nspec,ordrs[1]+1)
  tmpfit = x_setfitstrct(NITER=3L, NORD=ordnm, $
                    HSIG=2.5, LSIG=2.5, /FLGREJ)
  fitstr = replicate(tmpfit,nspec,ordrs[1]+1)
  
  ;; Loop on Exp. Number
  for kk=0L,nexp-1 do begin
    if nexp EQ 1 then begin
      for qq=ordrs[0],ordrs[1],ostep do begin
          ;; JXP
          all_ordr = where(mikeobj.order EQ qq, nobj)
          echfspec.phys_ordr[qq] = qq
          if nobj NE nspec then continue
          if nobj NE 1 then stop
          npix = mikeobj[all_ordr].npix
          echfspec.wave[0:npix-1,qq] = mikeobj[all_ordr].wave[0:npix-1]
          echfspec.fx[0:npix-1,qq] = mikeobj[all_ordr].flux[0:npix-1]
          echfspec.var[0:npix-1,qq] = (mikeobj[all_ordr].sig[0:npix-1])^2.
          ordr_typ[qq] = -1 
       endfor
       continue
    endif else begin
      for qq=ordrs[0],ordrs[1],ostep do begin
        all_ordr = where(mikeobj.order EQ qq, nobj)
        if nobj NE nspec then continue
        ;; Deal with FLUX vs not FLUX
        if keyword_set( NOFLUX ) then begin
            ;; Fill up flux array
            print, 'mike_combspec: WARNING -- Filling flux array with electrons'
            npix = mikeobj[all_ordr[kk]].npix
            mikeobj[all_ordr[kk]].flux[0:npix-1] = mikeobj[all_ordr[kk]].fx[0:npix-1] 
            mikeobj[all_ordr[kk]].sig[0:npix-1] = $
              sqrt(mikeobj[all_ordr[kk]].var[0:npix-1])
        endif
      endfor

      if kk EQ keyindx then continue

      gdsig = dindgen(5000L)*0.-1.
      gdnum = findgen(5000L)*0.

      ;; Loop on ORDER 
      for qq=ordrs[0],ordrs[1],ostep do begin
        all_ordr = where(mikeobj.order EQ qq, nobj)
        if nobj NE nspec then continue
        echfspec.phys_ordr[qq] = qq

        npix = 5000L
        ;; Find median ratio of flux and SNR
        tst = where(mikeobj[all_ordr[keyindx]].wave NE 0.)
        tstlen = size(tst,/dimensions)
        tstwv = mikeobj[all_ordr[keyindx]].wave
        tstwv[0:endtrm] = 0.
        tstwv[tstlen[0]-endtrm-1:tstlen[0]-1] = 0.
        highsnr = where(mikeobj[all_ordr[keyindx]].sig GT 0. AND $
                        mikeobj[all_ordr[keyindx]].flux / $
                        mikeobj[all_ordr[keyindx]].sig GT SNRMIN  AND $
                        mikeobj[all_ordr[kk]].sig GT 0. AND $
                        mikeobj[all_ordr[kk]].flux / $
                        mikeobj[all_ordr[kk]].sig GT SNRMIN AND $
                        tstwv NE 0., nhigh)
        if nhigh GT 100 then $
          med0 = median(mikeobj[all_ordr[keyindx]].flux[highsnr]$
                        /mikeobj[all_ordr[keyindx]].sig[highsnr]) $
        else med0 = 1.

        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; CALCULATE RATIO AND SNR
        if not keyword_set(SILENT) then begin
           print, 'mike_combspec: order = ', qq
           print, 'mike_combspec: nhigh = ', nhigh, ' med0 = ', med0
        endif
        if nhigh LT minpix1 then begin
            ;; Fill with exp. time ratio, may replace after other orders fit
            med_rto = mikeobj[all_ordr[keyindx]].exp/ $
              mikeobj[all_ordr[kk]].exp
            snr_rto = sqrt(mikeobj[all_ordr[keyindx]].exp/ $
              mikeobj[all_ordr[kk]].exp)
            svmedian[kk,qq] = med_rto
            svsnr[kk,qq] = snr_rto

            ordr_typ[kk,qq] = 0 
        endif else begin
            svfx = median(mikeobj[all_ordr[keyindx]].flux[highsnr],medindx)
            svsig = median(mikeobj[all_ordr[keyindx]].sig[highsnr],medindx)
            if nhigh LT 300 then begin
                ;; Take one value
                ;; Ratio
                rtio = (svfx/$
                        median(mikeobj[all_ordr[kk]].flux[highsnr],medindx))
                djs_iterstat, rtio, maxiter=7L, median=medv, sigrej=3.
                med_rto = medv
                    ;; SNR
                snr = mikeobj[all_ordr[kk]].flux[highsnr]/ $
                         mikeobj[all_ordr[kk]].sig[highsnr]/  (svfx/svsig)
                djs_iterstat, snr, maxiter=7L, median=medv, sigrej=3.
                snr_rto = medv
                ;; Save
                svmedian[kk,qq] = med_rto
                svsnr[kk,qq] = snr_rto
                ordr_typ[kk,qq] = 1 
            endif else begin
                ;; Fit a 2nd order POLY to the ratio 
                ;; Ratio
                rtio = (svfx /  $
                          median(mikeobj[all_ordr[kk]].flux[highsnr],medindx))
                fitwv = mikeobj[all_ordr[kk]].wave[highsnr]
                fit = x_fitrej(mikeobj[all_ordr[kk]].wave[highsnr], $
                               rtio, FITSTR=fitstr[kk,qq])
                med_rto = median(fit)
                ;if keyword_set(CHK) then begin
                ;  x_splot, rtio, ytwo=fit, /block, psym2=10, TITLE='rtio,fit'
                ;  stop
                ;endif
                ;; SNR
                snr = mikeobj[all_ordr[kk]].flux[highsnr]/ $
                  mikeobj[all_ordr[kk]].sig[highsnr]/ $
                  svfx * $
                  svsig
                djs_iterstat, snr, maxiter=7L, median=medv, sigrej=3.
                snr_rto = medv
                ;; Save
                svmedian[kk,qq] = med_rto
                svsnr[kk,qq] = snr_rto
                ordr_typ[kk,qq] = 2 
            endelse
        endelse
      endfor
    endelse
  endfor

  medianave = replicate(1.,nexp)
  mediansig = replicate(0.,nexp)
  if nexp NE 1 then begin
    for kk=0,nexp-1 do begin
      if kk EQ keyindx then continue
      for qq=ordrs[0],ordrs[1],ostep do begin
          ;; fix BAD orders
          if ordr_typ[kk,qq] EQ 0 then begin
            useordr = where(abs(lindgen(ordrs[1]+1)-qq) LE 5 $
                            AND ordr_typ[kk,*] GE 1, nuse)
            ord_diff = 10
            while nuse LE 5 AND ord_diff LE 35 do begin
                useordr = where(abs(lindgen(ordrs[1]+1)-qq) LE ord_diff $
                            AND ordr_typ[kk,*] GE 1, nuse)
                ord_diff = ord_diff + 5
            endwhile
            if nuse LT 5 then begin
              med_rto = mikeobj[all_ordr[keyindx]].exp/ $
                mikeobj[all_ordr[kk]].exp
              snr_rto = sqrt(mikeobj[all_ordr[keyindx]].exp/ $
                mikeobj[all_ordr[kk]].exp)
              svmedian[kk,qq] = med_rto
              svsnr[kk,qq] = snr_rto
            endif else begin
              svsnr[kk,qq] = median(svsnr[kk,useordr])
              svmedian[kk,qq] = median(svmedian[kk,useordr])
            endelse
          endif
      endfor 
      ;; Get Median, sig
      medianave[kk] = median(svmedian[kk,ordrs[0]:ordrs[1]])
      for qq=ordrs[0],ordrs[1],ostep do $
        mediansig[kk] = mediansig[kk] + (medianave[kk]-svmedian[kk,qq])^2.
      mediansig[kk] = sqrt(mediansig[kk]/(ordrs[1]-ordrs[0]+1))
      ;; Check for outliers
      if keyword_set(CHK) then begin
        xcol = lindgen(ordrs[1]-ordrs[0]+1)+ordrs[0]
        ptit = 'Median Ratios for Exposure ' + strtrim(string(kk),2) + $
                ' median = ' + strtrim(string(medianave[kk]),2) + $
                ' sigma = ' + strtrim(string(mediansig[kk]),2)
        x_splot, xcol, svmedian[kk,ordrs[0]:ordrs[1]], /block, TITLE=ptit
      endif
      for qq=ordrs[0],ordrs[1],ostep do begin
          if abs(svmedian[kk,qq] - medianave[kk]) GT 2.*mediansig[kk] then begin
              ordr_typ[kk,qq] = 0
              useordr = where(abs(lindgen(ordrs[1]+1)-qq) LE 5 $
                            AND ordr_typ[kk,*] GE 1, nuse)
              ord_diff = 10
              while nuse LE 5 AND ord_diff LE 35 do begin
                  useordr = where(abs(lindgen(ordrs[1]+1)-qq) LE ord_diff $
                              AND ordr_typ[kk,*] GE 1, nuse)
                  ord_diff = ord_diff + 5
              endwhile
              if nuse LT 5 then begin
                med_rto = mikeobj[all_ordr[keyindx]].exp/ $
                  mikeobj[all_ordr[kk]].exp
                snr_rto = sqrt(mikeobj[all_ordr[keyindx]].exp/ $
                  mikeobj[all_ordr[kk]].exp)
                svmedian[kk,qq] = med_rto
                svsnr[kk,qq] = snr_rto
              endif else begin
                svsnr[kk,qq] = median(svsnr[kk,useordr])
                svmedian[kk,qq] = median(svmedian[kk,useordr])
              endelse
          endif
      endfor
    endfor
    if keyword_set(CHK) then begin
        xcol = lindgen(ordrs[1]-ordrs[0]+1)+ordrs[0]
        for qq=0,nexp-1 do begin
            if qq EQ keyindx then continue
            ptit = 'Median Ratios for Exposure ' + strtrim(string(qq),2) + $
                    ' median = ' + strtrim(string(medianave[qq]),2) + $
                    ' sigma = ' + strtrim(string(mediansig[qq]),2)
            x_splot, xcol, svmedian[qq,ordrs[0]:ordrs[1]], /block, TITLE=ptit
        endfor
    endif

    for kk=0,nexp-1 do begin
      if kk EQ keyindx then continue
      for qq=ordrs[0],ordrs[1],ostep do begin
          all_ordr = where(mikeobj.order EQ qq, nobj)
          if nobj NE nspec then continue
          echfspec.phys_ordr[qq] = qq
          ;; Rescale
          if ordr_typ[kk,qq] EQ 1 OR ordr_typ[kk,qq] EQ 0 then begin
              mikeobj[all_ordr[kk]].flux = $
                mikeobj[all_ordr[kk]].flux * svmedian[kk,qq] 
              mikeobj[all_ordr[kk]].sig = $
                mikeobj[all_ordr[kk]].sig * svmedian[kk,qq]

              if keyword_set(CHK) then begin
                wuse = where(mikeobj[all_ordr[keyindx]].wave NE 0.)
                ptit = 'Rescaled 0, 1, Order: ' + strtrim(string(qq),2) + $
                       ' and Exposure (red): ' + strtrim(string(kk),2)
                x_splot, mikeobj[all_ordr[keyindx]].flux[wuse], TITLE=ptit,$
                   ytwo=mikeobj[all_ordr[kk]].flux[wuse], /block
              endif
          endif 
          if ordr_typ[kk,qq] EQ 2 then begin
              ;; Rescale
              tstwv = mikeobj[all_ordr[keyindx]].wave
              tstwv[0:endtrm] = 0.
              tstwv[tstlen[0]-endtrm-1:tstlen[0]-1] = 0.
              highsnr = where(mikeobj[all_ordr[keyindx]].sig GT 0. AND $
                              mikeobj[all_ordr[keyindx]].flux / $
                              mikeobj[all_ordr[keyindx]].sig GT SNRMIN  AND $
                              mikeobj[all_ordr[kk]].sig GT 0. AND $
                              mikeobj[all_ordr[kk]].flux / $
                              mikeobj[all_ordr[kk]].sig GT SNRMIN AND $
                              tstwv NE 0., nhigh)
              svfx = median(mikeobj[all_ordr[keyindx]].flux[highsnr],medindx)
              svsig = median(mikeobj[all_ordr[keyindx]].sig[highsnr],medindx)
              rtio = (svfx /  $
                        median(mikeobj[all_ordr[kk]].flux[highsnr],medindx))
              gdwv = where(mikeobj[all_ordr[kk]].wave GT 0.)
              fitwv = mikeobj[all_ordr[kk]].wave[highsnr]
              tmpfit = x_setfitstrct(NITER=3L, NORD=ordnm, $
                    HSIG=2.5, LSIG=2.5, /FLGREJ)
              fit = x_fitrej(mikeobj[all_ordr[kk]].wave[highsnr], $
                             rtio, FITSTR=tmpfit)
              if keyword_set(CHK) then begin
                ptit = 'RTIO,FIT for Order: ' + strtrim(string(qq),2) + $
                       ' and Exposure: ' + strtrim(string(kk),2)
                x_splot, highsnr, rtio, ytwo=fit, /block, psym2=10, TITLE=ptit
              ;  stop
              endif
              allfit = x_calcfit(mikeobj[all_ordr[kk]].wave[gdwv], $
                                 FITSTR=tmpfit)
                                 ;FITSTR=svfit[qq])
              mikeobj[all_ordr[kk]].flux[gdwv] = $
                mikeobj[all_ordr[kk]].flux[gdwv] * allfit
              mikeobj[all_ordr[kk]].sig[gdwv] = $
                mikeobj[all_ordr[kk]].sig[gdwv] * allfit

              if keyword_set(CHK) then begin
                wuse = where(mikeobj[all_ordr[keyindx]].wave NE 0.)
                ptit = 'Rescaled 2, Order: ' + strtrim(string(qq),2) + $
                       ' and Exposure (red): ' + strtrim(string(kk),2)
                x_splot, mikeobj[all_ordr[keyindx]].flux[wuse], TITLE=ptit, $
                   ytwo=mikeobj[all_ordr[kk]].flux[wuse], /block, psym2=10, psym1=10
              endif
          endif 
      endfor
    endfor

    for qq=ordrs[0],ordrs[1],ostep do begin
      all_ordr = where(mikeobj.order EQ qq, nobj)
      if nobj NE nspec then continue
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ;; Set bad pix to -1
      sig = double(mikeobj[all_ordr].sig[0:npix-1])
      a = where(mikeobj[all_ordr].sig[0:npix-1] LE 0., na)
      if na GT 0 then sig[a] = -1.
      if not keyword_set(SILENT) then print, 'mike_combspec: Order ', qq

      ;; FIX SIGMA FOR 2 EXPOSURE CASE
      if nobj EQ 2 then begin
         sigma = 0.
         gdvar = where(mikeobj[all_ordr[0]].sig[0:npix-1] GT 0. AND $
                       mikeobj[all_ordr[1]].sig[0:npix-1] GT 0., ngdvar)
         if ngdvar NE 0 then begin
             d = (mikeobj[all_ordr[0]].flux[gdvar]-mikeobj[all_ordr[1]].flux[gdvar])/ $
                 sqrt(mikeobj[all_ordr[0]].sig[gdvar]^2 + mikeobj[all_ordr[1]].sig[gdvar]^2)
	     djs_iterstat, d, sigma=sigma, sigrej=sigrej, maxiter=5
             sig = sig*sigma
             d = (mikeobj[all_ordr[0]].flux[gdvar]-mikeobj[all_ordr[1]].flux[gdvar])/ $
                 sqrt(sigma^2*mikeobj[all_ordr[0]].sig[gdvar]^2 + sigma^2*mikeobj[all_ordr[1]].sig[gdvar]^2)
	     djs_iterstat, d, sigma=sigma2, sigrej=sigrej, maxiter=5
         endif 
      endif 

      ;; COMBINE
      sz = size(mikeobj[all_ordr].flux[0:npix-1],/dimensions)
      mask = replicate(0,sz[0],sz[1])
      svmedian[*,qq] = 1.
;for i=0,nexp-1 do begin
;gdp = where(mikeobj[all_ordr[i]].wave GT 0.)
;x_splot, mikeobj[all_ordr[i]].wave[gdp], mikeobj[all_ordr[i]].flux[gdp], ytwo=mikeobj[all_ordr[i]].sig[gdp], /block, TITLE=strtrim(string(i),2)
;endfor

      x_combspec, mikeobj[all_ordr].flux[0:npix-1], sig^2, $
        fflux, fvar, WAVE=mikeobj[all_ordr[0]].wave[0:npix-1], $
        SNR=svsnr[*,qq], SCALE=svmedian[*,qq], mask=mask

      echfspec.wave[0:npix-1,qq] = mikeobj[all_ordr[0]].wave[0:npix-1]
      echfspec.fx[0:npix-1,qq] = temporary(fflux[0:npix-1])
      echfspec.var[0:npix-1,qq] = temporary(fvar[0:npix-1])
      bd = where(echfspec.wave[*,qq] LE 0,nbd)
      echfspec.var[bd,qq] = 0.

      ;; FIX SIGMA FOR > 2 EXPOSURE CASE
      if nobj GT 2 then begin
          usig = mikeobj[all_ordr[0]].sig[0:npix-1]
          bad = where(mask[*,0] EQ 0,nbad)
          if nbad GT 0 then usig[bad] = -1.
          gd = where(usig GT 0., ngd)
          ntot = ngd
          if ngd GT 0 then chi = (mikeobj[all_ordr[0]].flux[gd]-echfspec.fx[gd,qq])/usig
          for i=1, nobj-1 do begin
              usig = mikeobj[all_ordr[i]].sig[0:npix-1]
              bad = where(mask[*,i] EQ 0,nbad)
              if nbad GT 0 then usig[bad] = -1.
              gd = where(usig GT 0.,ngd)
              if ngd GT 0 AND ntot GT 0 then chi = [chi,(mikeobj[all_ordr[i]].flux[gd]-echfspec.fx[gd,qq])/usig]
              if ngd GT 0 AND ntot EQ 0 then chi = (mikeobj[all_ordr[i]].flux[gd]-echfspec.fx[gd,qq])/usig
              ntot = ntot+ngd
          endfor
          djs_iterstat, chi, sigma=sigma, sigrej=sigrej, maxiter=5
;;;TEST
         if ntot GT 0 then ochihist = histogram(chi,min=-5,max=5,binsize=0.1)
         usig = sigma*mikeobj[all_ordr[0]].sig[0:npix-1]
         bad = where(mask[*,0] EQ 0,nbad)
         if nbad GT 0 then usig[bad] = -1.
         ntot = 0
         gd = where(usig GT 0., ngd)
         ntot = ntot+ngd
         if ngd GT 0 then chi = (mikeobj[all_ordr[0]].flux[gd]-echfspec.fx[gd,qq])/usig
         for i=1, nobj-1 do begin
             usig = sigma*mikeobj[all_ordr[i]].sig[0:npix-1]
             bad = where(mask[*,i] EQ 0,nbad)
             if nbad GT 0 then usig[bad] = -1.
             gd = where(usig GT 0.,ngd)
             ntot = ntot+ngd
             if ngd GT 0 then chi = [chi,(mikeobj[all_ordr[i]].flux[gd]-echfspec.fx[gd,qq])/usig]
         endfor
         if ntot GT 0 then begin
             chihist = histogram(chi,min=-5,max=5,binsize=0.1)
             djs_iterstat, chi, sigma=sigma2, sigrej=sigrej, maxiter=5
             title = 'Sigma Original: ' + strtrim(string(sigma),2) + $
                     ' New Sigma: ' + strtrim(string(sigma2),2)
          ;   x_splot, findgen(101)*0.1-5, ochihist, ytwo=chihist, /block, TITLE=title

          x_combspec, mikeobj[all_ordr].flux[0:npix-1], (sig*sigma)^2, $
            nflux, nvar, WAVE=mikeobj[all_ordr[0]].wave[0:npix-1], $
            SNR=svsnr[*,qq], SCALE=svmedian[*,qq], mask=mask
          gdp = where(echfspec.wave[0:npix-1,qq] GT 0.,ngdp)
          ;x_splot, echfspec.wave[gdp,qq], fflux[gdp], ytwo=nflux[gdp], /block, TITLE='Flux'
          ;x_splot, echfspec.wave[gdp,qq], sqrt(fvar[gdp]), ytwo=sqrt(nvar[gdp]), /block, TITLE='sigma'
          echfspec.fx[0:npix-1,qq] = temporary(nflux[0:npix-1])
          echfspec.var[0:npix-1,qq] = temporary(nvar[0:npix-1])
         endif
      endif 

   if keyword_set( JNK ) then begin
      a = where(mikeobj[all_ordr[0]].sig[0:npix-1] GT 0., na)
      if na EQ 0 then continue
      gdsig = gdsig*0.-1.
      gdnum = gdnum*0.
      gdsig[a] = (echfspec.fx[a,qq]-mikeobj[all_ordr[0]].flux[a])^2
      gdnum[a] = gdnum[a]+1.
      for ss=1, nobj-1 do begin
          a = where(mikeobj[all_ordr[ss]].sig[0:npix-1] GT 0., na)
          if na EQ 0 then stop
          gdsig[a] = gdsig[a]+(echfspec.fx[a,qq]-mikeobj[all_ordr[ss]].flux[a])^2
          gdnum[a] = gdnum[a]+1.
      endfor
      a = where(gdnum GT 1.,na)
      if na NE 0 then gdsig[a] = gdsig[a]/(gdnum[a]-1.)
      a = where(gdsig GT 0.,na)
      if na NE 0 then gdsig[a] = sqrt(gdsig[a])
      a = where(gdnum LE 1.,na)
      if na NE 0 then gdsig[a] = sqrt(echfspec.var[a,qq])

      for ss=0, nobj-1 do begin
          a = where(mikeobj[all_ordr[ss]].sig[0:npix-1] GT 0., na)
          if na EQ 0 then stop
          diffi = (echfspec.fx[a,qq]-mikeobj[all_ordr[ss]].flux[a])
          diff = diffi/sqrt(echfspec.var[a,qq]+mikeobj[all_ordr[ss]].sig[a]^2)
          diffn = diffi/sqrt(gdsig[a]^2+mikeobj[all_ordr[ss]].sig[a]^2)
          dh = histogram(diff,min=-4,max=4,binsize=0.01) 
	djs_iterstat, diff, sigma=sigma, sigrej=sigrej, maxiter=5
	if not keyword_set(SILENT) then print, sigma
;          dhn = histogram(diffn,min=-4,max=4,binsize=0.01) 
          ;x_splot, dindgen(801)*0.01-4., dh, /block
      endfor
    endif
      ;;
      if keyword_set( CHK ) then begin
          wuse = where(echfspec.wave[*,qq] NE 0.)
          x_splot, echfspec.wave[wuse,qq], echfspec.fx[wuse,qq], $
             TITLE='Combined', /block
          ;  ytwo=mikeobj[all_ordr[0]].flux[wuse], TITLE='Combined', $
          ;  ythr=mikeobj[all_ordr[1]].flux[wuse], psym3=-3, /block
      endif
    endfor
  endif 
      ;; Output
;      mike_echcomb_out, echfspec
              

;;;; OUTPUT  ;;;;

  mike_wrfspec, echfspec, outfil 

;  close, 56

  print, 'mike_combspec:  Output is in ', outfil
  print, 'mike_combspec:  All done!'


  return
end
  

